@@ -1,9 +1,32 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2014 Google Inc.  All rights reserved.
+// https://developers.google.com/protocol-buffers/
 //
-// Use of this source code is governed by a BSD-style
-// license that can be found in the LICENSE file or at
-// https://developers.google.com/open-source/licenses/bsd
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are
+// met:
+//
+//     * Redistributions of source code must retain the above copyright
+// notice, this list of conditions and the following disclaimer.
+//     * Redistributions in binary form must reproduce the above
+// copyright notice, this list of conditions and the following disclaimer
+// in the documentation and/or other materials provided with the
+// distribution.
+//     * Neither the name of Google Inc. nor the names of its
+// contributors may be used to endorse or promote products derived from
+// this software without specific prior written permission.
+//
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <ctype.h>
 #include <errno.h>
@@ -44,14 +67,12 @@ static VALUE rb_str_maybe_null(const char* s) {
   }
   return rb_str_new2(s);
 }
-static ID options_instancevar_interned;
+
 // -----------------------------------------------------------------------------
 // DescriptorPool.
 // -----------------------------------------------------------------------------
 
 typedef struct {
-  // IMPORTANT: WB_PROTECTED objects must only use the RB_OBJ_WRITE()
-  // macro to update VALUE references, as to trigger write barriers.
   VALUE def_to_descriptor;  // Hash table of def* -> Ruby descriptor.
   upb_DefPool* symtab;
 } DescriptorPool;
@@ -76,7 +97,7 @@ static void DescriptorPool_free(void* _self) {
 static const rb_data_type_t DescriptorPool_type = {
     "Google::Protobuf::DescriptorPool",
     {DescriptorPool_mark, DescriptorPool_free, NULL},
-    .flags = RUBY_TYPED_FREE_IMMEDIATELY | RUBY_TYPED_WB_PROTECTED,
+    .flags = RUBY_TYPED_FREE_IMMEDIATELY,
 };
 
 static DescriptorPool* ruby_to_DescriptorPool(VALUE val) {
@@ -104,9 +125,11 @@ static VALUE DescriptorPool_alloc(VALUE klass) {
   self->def_to_descriptor = Qnil;
   ret = TypedData_Wrap_Struct(klass, &DescriptorPool_type, self);
 
-  RB_OBJ_WRITE(ret, &self->def_to_descriptor, rb_hash_new());
+  self->def_to_descriptor = rb_hash_new();
   self->symtab = upb_DefPool_New();
-  return ObjectCache_TryAdd(self->symtab, ret);
+  ObjectCache_Add(self->symtab, ret);
+
+  return ret;
 }
 
 /*
@@ -144,26 +167,20 @@ VALUE DescriptorPool_add_serialized_file(VALUE _self,
  * call-seq:
  *     DescriptorPool.lookup(name) => descriptor
  *
- * Finds a Descriptor, EnumDescriptor or FieldDescriptor by name and returns it,
- * or nil if none exists with the given name.
+ * Finds a Descriptor or EnumDescriptor by name and returns it, or nil if none
+ * exists with the given name.
  */
 static VALUE DescriptorPool_lookup(VALUE _self, VALUE name) {
   DescriptorPool* self = ruby_to_DescriptorPool(_self);
   const char* name_str = get_str(name);
   const upb_MessageDef* msgdef;
   const upb_EnumDef* enumdef;
-  const upb_FieldDef* fielddef;
 
   msgdef = upb_DefPool_FindMessageByName(self->symtab, name_str);
   if (msgdef) {
     return get_msgdef_obj(_self, msgdef);
   }
 
-  fielddef = upb_DefPool_FindExtensionByName(self->symtab, name_str);
-  if (fielddef) {
-    return get_fielddef_obj(_self, fielddef);
-  }
-
   enumdef = upb_DefPool_FindEnumByName(self->symtab, name_str);
   if (enumdef) {
     return get_enumdef_obj(_self, enumdef);
@@ -198,7 +215,6 @@ static void DescriptorPool_register(VALUE module) {
 
   rb_gc_register_address(&generated_pool);
   generated_pool = rb_class_new_instance(0, NULL, klass);
-  options_instancevar_interned = rb_intern("options");
 }
 
 // -----------------------------------------------------------------------------
@@ -207,8 +223,6 @@ static void DescriptorPool_register(VALUE module) {
 
 typedef struct {
   const upb_MessageDef* msgdef;
-  // IMPORTANT: WB_PROTECTED objects must only use the RB_OBJ_WRITE()
-  // macro to update VALUE references, as to trigger write barriers.
   VALUE klass;
   VALUE descriptor_pool;
 } Descriptor;
@@ -224,7 +238,7 @@ static void Descriptor_mark(void* _self) {
 static const rb_data_type_t Descriptor_type = {
     "Google::Protobuf::Descriptor",
     {Descriptor_mark, RUBY_DEFAULT_FREE, NULL},
-    .flags = RUBY_TYPED_FREE_IMMEDIATELY | RUBY_TYPED_WB_PROTECTED,
+    .flags = RUBY_TYPED_FREE_IMMEDIATELY,
 };
 
 static Descriptor* ruby_to_Descriptor(VALUE val) {
@@ -233,35 +247,6 @@ static Descriptor* ruby_to_Descriptor(VALUE val) {
   return ret;
 }
 
-// Decode and return a frozen instance of a Descriptor Option for the given pool
-static VALUE decode_options(VALUE self, const char* option_type, int size,
-                            const char* bytes, VALUE descriptor_pool) {
-  VALUE options_rb = rb_ivar_get(self, options_instancevar_interned);
-  if (options_rb != Qnil) {
-    return options_rb;
-  }
-
-  static const char* prefix = "google.protobuf.";
-  char fullname
-      [/*strlen(prefix)*/ 16 +
-       /*strln(longest option type supported e.g. "MessageOptions")*/ 14 +
-       /*null terminator*/ 1];
-
-  snprintf(fullname, sizeof(fullname), "%s%s", prefix, option_type);
-  const upb_MessageDef* msgdef = upb_DefPool_FindMessageByName(
-      ruby_to_DescriptorPool(descriptor_pool)->symtab, fullname);
-  if (!msgdef) {
-    rb_raise(rb_eRuntimeError, "Cannot find %s in DescriptorPool", option_type);
-  }
-
-  VALUE desc_rb = get_msgdef_obj(descriptor_pool, msgdef);
-  const Descriptor* desc = ruby_to_Descriptor(desc_rb);
-
-  options_rb = Message_decode_bytes(size, bytes, 0, desc->klass, true);
-  rb_ivar_set(self, options_instancevar_interned, options_rb);
-  return options_rb;
-}
-
 /*
  * call-seq:
  *     Descriptor.new => descriptor
@@ -295,7 +280,7 @@ static VALUE Descriptor_initialize(VALUE _self, VALUE cookie,
              "Descriptor objects may not be created from Ruby.");
   }
 
-  RB_OBJ_WRITE(_self, &self->descriptor_pool, descriptor_pool);
+  self->descriptor_pool = descriptor_pool;
   self->msgdef = (const upb_MessageDef*)NUM2ULL(ptr);
 
   return Qnil;
@@ -405,31 +390,11 @@ static VALUE Descriptor_lookup_oneof(VALUE _self, VALUE name) {
 static VALUE Descriptor_msgclass(VALUE _self) {
   Descriptor* self = ruby_to_Descriptor(_self);
   if (self->klass == Qnil) {
-    RB_OBJ_WRITE(_self, &self->klass, build_class_from_descriptor(_self));
+    self->klass = build_class_from_descriptor(_self);
   }
   return self->klass;
 }
 
-/*
- * call-seq:
- *     Descriptor.options => options
- *
- * Returns the `MessageOptions` for this `Descriptor`.
- */
-static VALUE Descriptor_options(VALUE _self) {
-  Descriptor* self = ruby_to_Descriptor(_self);
-  const google_protobuf_MessageOptions* opts =
-      upb_MessageDef_Options(self->msgdef);
-  upb_Arena* arena = upb_Arena_New();
-  size_t size;
-  char* serialized =
-      google_protobuf_MessageOptions_serialize(opts, arena, &size);
-  VALUE message_options = decode_options(_self, "MessageOptions", size,
-                                         serialized, self->descriptor_pool);
-  upb_Arena_Free(arena);
-  return message_options;
-}
-
 static void Descriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "Descriptor", rb_cObject);
   rb_define_alloc_func(klass, Descriptor_alloc);
@@ -441,7 +406,6 @@ static void Descriptor_register(VALUE module) {
   rb_define_method(klass, "msgclass", Descriptor_msgclass, 0);
   rb_define_method(klass, "name", Descriptor_name, 0);
   rb_define_method(klass, "file_descriptor", Descriptor_file_descriptor, 0);
-  rb_define_method(klass, "options", Descriptor_options, 0);
   rb_include_module(klass, rb_mEnumerable);
   rb_gc_register_address(&cDescriptor);
   cDescriptor = klass;
@@ -453,8 +417,6 @@ static void Descriptor_register(VALUE module) {
 
 typedef struct {
   const upb_FileDef* filedef;
-  // IMPORTANT: WB_PROTECTED objects must only use the RB_OBJ_WRITE()
-  // macro to update VALUE references, as to trigger write barriers.
   VALUE descriptor_pool;  // Owns the upb_FileDef.
 } FileDescriptor;
 
@@ -468,7 +430,7 @@ static void FileDescriptor_mark(void* _self) {
 static const rb_data_type_t FileDescriptor_type = {
     "Google::Protobuf::FileDescriptor",
     {FileDescriptor_mark, RUBY_DEFAULT_FREE, NULL},
-    .flags = RUBY_TYPED_FREE_IMMEDIATELY | RUBY_TYPED_WB_PROTECTED,
+    .flags = RUBY_TYPED_FREE_IMMEDIATELY,
 };
 
 static FileDescriptor* ruby_to_FileDescriptor(VALUE val) {
@@ -501,7 +463,7 @@ static VALUE FileDescriptor_initialize(VALUE _self, VALUE cookie,
              "Descriptor objects may not be created from Ruby.");
   }
 
-  RB_OBJ_WRITE(_self, &self->descriptor_pool, descriptor_pool);
+  self->descriptor_pool = descriptor_pool;
   self->filedef = (const upb_FileDef*)NUM2ULL(ptr);
 
   return Qnil;
@@ -541,31 +503,12 @@ static VALUE FileDescriptor_syntax(VALUE _self) {
   }
 }
 
-/*
- * call-seq:
- *     FileDescriptor.options => options
- *
- * Returns the `FileOptions` for this `FileDescriptor`.
- */
-static VALUE FileDescriptor_options(VALUE _self) {
-  FileDescriptor* self = ruby_to_FileDescriptor(_self);
-  const google_protobuf_FileOptions* opts = upb_FileDef_Options(self->filedef);
-  upb_Arena* arena = upb_Arena_New();
-  size_t size;
-  char* serialized = google_protobuf_FileOptions_serialize(opts, arena, &size);
-  VALUE file_options = decode_options(_self, "FileOptions", size, serialized,
-                                      self->descriptor_pool);
-  upb_Arena_Free(arena);
-  return file_options;
-}
-
 static void FileDescriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "FileDescriptor", rb_cObject);
   rb_define_alloc_func(klass, FileDescriptor_alloc);
   rb_define_method(klass, "initialize", FileDescriptor_initialize, 3);
   rb_define_method(klass, "name", FileDescriptor_name, 0);
   rb_define_method(klass, "syntax", FileDescriptor_syntax, 0);
-  rb_define_method(klass, "options", FileDescriptor_options, 0);
   rb_gc_register_address(&cFileDescriptor);
   cFileDescriptor = klass;
 }
@@ -576,8 +519,6 @@ static void FileDescriptor_register(VALUE module) {
 
 typedef struct {
   const upb_FieldDef* fielddef;
-  // IMPORTANT: WB_PROTECTED objects must only use the RB_OBJ_WRITE()
-  // macro to update VALUE references, as to trigger write barriers.
   VALUE descriptor_pool;  // Owns the upb_FieldDef.
 } FieldDescriptor;
 
@@ -591,7 +532,7 @@ static void FieldDescriptor_mark(void* _self) {
 static const rb_data_type_t FieldDescriptor_type = {
     "Google::Protobuf::FieldDescriptor",
     {FieldDescriptor_mark, RUBY_DEFAULT_FREE, NULL},
-    .flags = RUBY_TYPED_FREE_IMMEDIATELY | RUBY_TYPED_WB_PROTECTED,
+    .flags = RUBY_TYPED_FREE_IMMEDIATELY,
 };
 
 static FieldDescriptor* ruby_to_FieldDescriptor(VALUE val) {
@@ -616,7 +557,7 @@ static VALUE FieldDescriptor_alloc(VALUE klass) {
 
 /*
  * call-seq:
- *    FieldDescriptor.new(c_only_cookie, pool, ptr) => FieldDescriptor
+ *    EnumDescriptor.new(c_only_cookie, pool, ptr) => EnumDescriptor
  *
  * Creates a descriptor wrapper object.  May only be called from C.
  */
@@ -629,7 +570,7 @@ static VALUE FieldDescriptor_initialize(VALUE _self, VALUE cookie,
              "Descriptor objects may not be created from Ruby.");
   }
 
-  RB_OBJ_WRITE(_self, &self->descriptor_pool, descriptor_pool);
+  self->descriptor_pool = descriptor_pool;
   self->fielddef = (const upb_FieldDef*)NUM2ULL(ptr);
 
   return Qnil;
@@ -654,7 +595,7 @@ upb_CType ruby_to_fieldtype(VALUE type) {
 
 #define CONVERT(upb, ruby)                \
   if (SYM2ID(type) == rb_intern(#ruby)) { \
-    return kUpb_CType_##upb;              \
+    return kUpb_CType_##upb;                \
   }
 
   CONVERT(Float, float);
@@ -677,7 +618,7 @@ upb_CType ruby_to_fieldtype(VALUE type) {
 
 static VALUE descriptortype_to_ruby(upb_FieldType type) {
   switch (type) {
-#define CONVERT(upb, ruby)   \
+#define CONVERT(upb, ruby)        \
   case kUpb_FieldType_##upb: \
     return ID2SYM(rb_intern(#ruby));
     CONVERT(Float, float);
@@ -762,7 +703,7 @@ static VALUE FieldDescriptor_label(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
   switch (upb_FieldDef_Label(self->fielddef)) {
 #define CONVERT(upb, ruby) \
-  case kUpb_Label_##upb:   \
+  case kUpb_Label_##upb:    \
     return ID2SYM(rb_intern(#ruby));
 
     CONVERT(Optional, optional);
@@ -862,7 +803,7 @@ static VALUE FieldDescriptor_get(VALUE _self, VALUE msg_rb) {
 static VALUE FieldDescriptor_has(VALUE _self, VALUE msg_rb) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
   const upb_MessageDef* m;
-  const upb_Message* msg = Message_Get(msg_rb, &m);
+  const upb_MessageDef* msg = Message_Get(msg_rb, &m);
 
   if (m != upb_FieldDef_ContainingType(self->fielddef)) {
     rb_raise(cTypeError, "has method called on wrong message type");
@@ -882,7 +823,7 @@ static VALUE FieldDescriptor_has(VALUE _self, VALUE msg_rb) {
 static VALUE FieldDescriptor_clear(VALUE _self, VALUE msg_rb) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
   const upb_MessageDef* m;
-  upb_Message* msg = Message_GetMutable(msg_rb, &m);
+  upb_MessageDef* msg = Message_GetMutable(msg_rb, &m);
 
   if (m != upb_FieldDef_ContainingType(self->fielddef)) {
     rb_raise(cTypeError, "has method called on wrong message type");
@@ -903,7 +844,7 @@ static VALUE FieldDescriptor_clear(VALUE _self, VALUE msg_rb) {
 static VALUE FieldDescriptor_set(VALUE _self, VALUE msg_rb, VALUE value) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
   const upb_MessageDef* m;
-  upb_Message* msg = Message_GetMutable(msg_rb, &m);
+  upb_MessageDef* msg = Message_GetMutable(msg_rb, &m);
   upb_Arena* arena = Arena_get(Message_GetArena(msg_rb));
   upb_MessageValue msgval;
 
@@ -917,25 +858,6 @@ static VALUE FieldDescriptor_set(VALUE _self, VALUE msg_rb, VALUE value) {
   return Qnil;
 }
 
-/*
- * call-seq:
- *     FieldDescriptor.options => options
- *
- * Returns the `FieldOptions` for this `FieldDescriptor`.
- */
-static VALUE FieldDescriptor_options(VALUE _self) {
-  FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
-  const google_protobuf_FieldOptions* opts =
-      upb_FieldDef_Options(self->fielddef);
-  upb_Arena* arena = upb_Arena_New();
-  size_t size;
-  char* serialized = google_protobuf_FieldOptions_serialize(opts, arena, &size);
-  VALUE field_options = decode_options(_self, "FieldOptions", size, serialized,
-                                       self->descriptor_pool);
-  upb_Arena_Free(arena);
-  return field_options;
-}
-
 static void FieldDescriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "FieldDescriptor", rb_cObject);
   rb_define_alloc_func(klass, FieldDescriptor_alloc);
@@ -952,7 +874,6 @@ static void FieldDescriptor_register(VALUE module) {
   rb_define_method(klass, "clear", FieldDescriptor_clear, 1);
   rb_define_method(klass, "get", FieldDescriptor_get, 1);
   rb_define_method(klass, "set", FieldDescriptor_set, 2);
-  rb_define_method(klass, "options", FieldDescriptor_options, 0);
   rb_gc_register_address(&cFieldDescriptor);
   cFieldDescriptor = klass;
 }
@@ -963,8 +884,6 @@ static void FieldDescriptor_register(VALUE module) {
 
 typedef struct {
   const upb_OneofDef* oneofdef;
-  // IMPORTANT: WB_PROTECTED objects must only use the RB_OBJ_WRITE()
-  // macro to update VALUE references, as to trigger write barriers.
   VALUE descriptor_pool;  // Owns the upb_OneofDef.
 } OneofDescriptor;
 
@@ -978,7 +897,7 @@ static void OneofDescriptor_mark(void* _self) {
 static const rb_data_type_t OneofDescriptor_type = {
     "Google::Protobuf::OneofDescriptor",
     {OneofDescriptor_mark, RUBY_DEFAULT_FREE, NULL},
-    .flags = RUBY_TYPED_FREE_IMMEDIATELY | RUBY_TYPED_WB_PROTECTED,
+    .flags = RUBY_TYPED_FREE_IMMEDIATELY,
 };
 
 static OneofDescriptor* ruby_to_OneofDescriptor(VALUE val) {
@@ -1017,7 +936,7 @@ static VALUE OneofDescriptor_initialize(VALUE _self, VALUE cookie,
              "Descriptor objects may not be created from Ruby.");
   }
 
-  RB_OBJ_WRITE(_self, &self->descriptor_pool, descriptor_pool);
+  self->descriptor_pool = descriptor_pool;
   self->oneofdef = (const upb_OneofDef*)NUM2ULL(ptr);
 
   return Qnil;
@@ -1052,32 +971,12 @@ static VALUE OneofDescriptor_each(VALUE _self) {
   return Qnil;
 }
 
-/*
- * call-seq:
- *     OneofDescriptor.options => options
- *
- * Returns the `OneofOptions` for this `OneofDescriptor`.
- */
-static VALUE OneOfDescriptor_options(VALUE _self) {
-  OneofDescriptor* self = ruby_to_OneofDescriptor(_self);
-  const google_protobuf_OneofOptions* opts =
-      upb_OneofDef_Options(self->oneofdef);
-  upb_Arena* arena = upb_Arena_New();
-  size_t size;
-  char* serialized = google_protobuf_OneofOptions_serialize(opts, arena, &size);
-  VALUE oneof_options = decode_options(_self, "OneofOptions", size, serialized,
-                                       self->descriptor_pool);
-  upb_Arena_Free(arena);
-  return oneof_options;
-}
-
 static void OneofDescriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "OneofDescriptor", rb_cObject);
   rb_define_alloc_func(klass, OneofDescriptor_alloc);
   rb_define_method(klass, "initialize", OneofDescriptor_initialize, 3);
   rb_define_method(klass, "name", OneofDescriptor_name, 0);
   rb_define_method(klass, "each", OneofDescriptor_each, 0);
-  rb_define_method(klass, "options", OneOfDescriptor_options, 0);
   rb_include_module(klass, rb_mEnumerable);
   rb_gc_register_address(&cOneofDescriptor);
   cOneofDescriptor = klass;
@@ -1089,8 +988,6 @@ static void OneofDescriptor_register(VALUE module) {
 
 typedef struct {
   const upb_EnumDef* enumdef;
-  // IMPORTANT: WB_PROTECTED objects must only use the RB_OBJ_WRITE()
-  // macro to update VALUE references, as to trigger write barriers.
   VALUE module;           // begins as nil
   VALUE descriptor_pool;  // Owns the upb_EnumDef.
 } EnumDescriptor;
@@ -1106,7 +1003,7 @@ static void EnumDescriptor_mark(void* _self) {
 static const rb_data_type_t EnumDescriptor_type = {
     "Google::Protobuf::EnumDescriptor",
     {EnumDescriptor_mark, RUBY_DEFAULT_FREE, NULL},
-    .flags = RUBY_TYPED_FREE_IMMEDIATELY | RUBY_TYPED_WB_PROTECTED,
+    .flags = RUBY_TYPED_FREE_IMMEDIATELY,
 };
 
 static EnumDescriptor* ruby_to_EnumDescriptor(VALUE val) {
@@ -1145,7 +1042,7 @@ static VALUE EnumDescriptor_initialize(VALUE _self, VALUE cookie,
              "Descriptor objects may not be created from Ruby.");
   }
 
-  RB_OBJ_WRITE(_self, &self->descriptor_pool, descriptor_pool);
+  self->descriptor_pool = descriptor_pool;
   self->enumdef = (const upb_EnumDef*)NUM2ULL(ptr);
 
   return Qnil;
@@ -1184,7 +1081,7 @@ static VALUE EnumDescriptor_name(VALUE _self) {
 static VALUE EnumDescriptor_lookup_name(VALUE _self, VALUE name) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
   const char* name_str = rb_id2name(SYM2ID(name));
-  const upb_EnumValueDef* ev =
+  const upb_EnumValueDef *ev =
       upb_EnumDef_FindValueByName(self->enumdef, name_str);
   if (ev) {
     return INT2NUM(upb_EnumValueDef_Number(ev));
@@ -1203,8 +1100,7 @@ static VALUE EnumDescriptor_lookup_name(VALUE _self, VALUE name) {
 static VALUE EnumDescriptor_lookup_value(VALUE _self, VALUE number) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
   int32_t val = NUM2INT(number);
-  const upb_EnumValueDef* ev =
-      upb_EnumDef_FindValueByNumber(self->enumdef, val);
+  const upb_EnumValueDef* ev = upb_EnumDef_FindValueByNumber(self->enumdef, val);
   if (ev) {
     return ID2SYM(rb_intern(upb_EnumValueDef_Name(ev)));
   } else {
@@ -1242,29 +1138,11 @@ static VALUE EnumDescriptor_each(VALUE _self) {
 static VALUE EnumDescriptor_enummodule(VALUE _self) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
   if (self->module == Qnil) {
-    RB_OBJ_WRITE(_self, &self->module, build_module_from_enumdesc(_self));
+    self->module = build_module_from_enumdesc(_self);
   }
   return self->module;
 }
 
-/*
- * call-seq:
- *     EnumDescriptor.options => options
- *
- * Returns the `EnumOptions` for this `EnumDescriptor`.
- */
-static VALUE EnumDescriptor_options(VALUE _self) {
-  EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
-  const google_protobuf_EnumOptions* opts = upb_EnumDef_Options(self->enumdef);
-  upb_Arena* arena = upb_Arena_New();
-  size_t size;
-  char* serialized = google_protobuf_EnumOptions_serialize(opts, arena, &size);
-  VALUE enum_options = decode_options(_self, "EnumOptions", size, serialized,
-                                      self->descriptor_pool);
-  upb_Arena_Free(arena);
-  return enum_options;
-}
-
 static void EnumDescriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "EnumDescriptor", rb_cObject);
   rb_define_alloc_func(klass, EnumDescriptor_alloc);
@@ -1275,7 +1153,6 @@ static void EnumDescriptor_register(VALUE module) {
   rb_define_method(klass, "each", EnumDescriptor_each, 0);
   rb_define_method(klass, "enummodule", EnumDescriptor_enummodule, 0);
   rb_define_method(klass, "file_descriptor", EnumDescriptor_file_descriptor, 0);
-  rb_define_method(klass, "options", EnumDescriptor_options, 0);
   rb_include_module(klass, rb_mEnumerable);
   rb_gc_register_address(&cEnumDescriptor);
   cEnumDescriptor = klass;
