@@ -28,48 +28,40 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#include <google/protobuf/compiler/php/php_generator.h>
-
-#include <google/protobuf/compiler/code_generator.h>
-#include <google/protobuf/compiler/plugin.h>
-#include <google/protobuf/descriptor.h>
-#include <google/protobuf/descriptor.pb.h>
-#include <google/protobuf/io/printer.h>
-#include <google/protobuf/io/zero_copy_stream.h>
-#include <google/protobuf/stubs/strutil.h>
+#include "google/protobuf/compiler/php/php_generator.h"
 
+#include <algorithm>
 #include <sstream>
+#include <string>
+#include <vector>
 
-const std::string kDescriptorFile = "google/protobuf/descriptor.proto";
-const std::string kEmptyFile = "google/protobuf/empty.proto";
-const std::string kEmptyMetadataFile = "GPBMetadata/Google/Protobuf/GPBEmpty.php";
-const std::string kDescriptorMetadataFile =
+#include "google/protobuf/compiler/code_generator.h"
+#include "absl/container/flat_hash_map.h"
+#include "absl/container/flat_hash_set.h"
+#include "absl/log/absl_log.h"
+#include "absl/strings/ascii.h"
+#include "absl/strings/escaping.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_replace.h"
+#include "absl/strings/str_split.h"
+#include "absl/strings/string_view.h"
+#include "google/protobuf/descriptor.h"
+#include "google/protobuf/descriptor.pb.h"
+#include "google/protobuf/io/printer.h"
+#include "google/protobuf/io/zero_copy_stream.h"
+
+constexpr absl::string_view kDescriptorFile =
+    "google/protobuf/descriptor.proto";
+constexpr absl::string_view kEmptyFile = "google/protobuf/empty.proto";
+constexpr absl::string_view kEmptyMetadataFile =
+    "GPBMetadata/Google/Protobuf/GPBEmpty.php";
+constexpr absl::string_view kDescriptorMetadataFile =
     "GPBMetadata/Google/Protobuf/Internal/Descriptor.php";
-const std::string kDescriptorDirName = "Google/Protobuf/Internal";
-const std::string kDescriptorPackageName = "Google\\Protobuf\\Internal";
-const char* const kReservedNames[] = {
-    "abstract",     "and",        "array",        "as",         "break",
-    "callable",     "case",       "catch",        "class",      "clone",
-    "const",        "continue",   "declare",      "default",    "die",
-    "do",           "echo",       "else",         "elseif",     "empty",
-    "enddeclare",   "endfor",     "endforeach",   "endif",      "endswitch",
-    "endwhile",     "eval",       "exit",         "extends",    "final",
-    "finally",      "fn",         "for",          "foreach",    "function",
-    "global",       "goto",       "if",           "implements", "include",
-    "include_once", "instanceof", "insteadof",    "interface",  "isset",
-    "list",         "match",      "namespace",    "new",        "or",
-    "parent",       "print",      "private",      "protected",  "public",
-    "readonly",     "require",    "require_once", "return",     "self",
-    "static",       "switch",     "throw",        "trait",      "try",
-    "unset",        "use",        "var",          "while",      "xor",
-    "yield",        "int",        "float",        "bool",       "string",
-    "true",         "false",      "null",         "void",       "iterable"};
-const char* const kValidConstantNames[] = {
-    "int",   "float", "bool", "string",   "true",
-    "false", "null",  "void", "iterable", "parent",
-    "self", "readonly"
-};
-const int kReservedNamesSize = 80;
+constexpr absl::string_view kDescriptorPackageName =
+    "Google\\Protobuf\\Internal";
+constexpr absl::string_view kValidConstantNames[] = {
+    "int",  "float", "bool",     "string", "true", "false",
+    "null", "void",  "iterable", "parent", "self", "readonly"};
 const int kValidConstantNamesSize = 12;
 const int kFieldSetter = 1;
 const int kFieldGetter = 2;
@@ -84,18 +76,18 @@ struct Options {
   bool is_descriptor = false;
   bool aggregate_metadata = false;
   bool gen_c_wkt = false;
-  std::set<std::string> aggregate_metadata_prefixes;
+  absl::flat_hash_set<std::string> aggregate_metadata_prefixes;
 };
 
 namespace {
 
 // Forward decls.
-std::string PhpName(const std::string& full_name, const Options& options);
+std::string PhpName(absl::string_view full_name, const Options& options);
 std::string IntToString(int32_t value);
-std::string FilenameToClassname(const std::string& filename);
+std::string FilenameToClassname(absl::string_view filename);
 std::string GeneratedMetadataFileName(const FileDescriptor* file,
                                       const Options& options);
-std::string UnderscoresToCamelCase(const std::string& name,
+std::string UnderscoresToCamelCase(absl::string_view name,
                                    bool cap_first_letter);
 void Indent(io::Printer* printer);
 void Outdent(io::Printer* printer);
@@ -121,68 +113,19 @@ void GenerateServiceDocComment(io::Printer* printer,
 void GenerateServiceMethodDocComment(io::Printer* printer,
                               const MethodDescriptor* method);
 
-std::string ReservedNamePrefix(const std::string& classname,
-                                const FileDescriptor* file) {
-  bool is_reserved = false;
-
-  std::string lower = classname;
-  std::transform(lower.begin(), lower.end(), lower.begin(), ::tolower);
-
-  for (int i = 0; i < kReservedNamesSize; i++) {
-    if (lower == kReservedNames[i]) {
-      is_reserved = true;
-      break;
-    }
-  }
-
-  if (is_reserved) {
-    if (file->package() == "google.protobuf") {
-      return "GPB";
-    } else {
-      return "PB";
-    }
-  }
-
-  return "";
-}
-
 template <typename DescriptorType>
 std::string DescriptorFullName(const DescriptorType* desc, bool is_internal) {
+  absl::string_view full_name = desc->full_name();
   if (is_internal) {
-    return StringReplace(desc->full_name(),
-                         "google.protobuf",
-                         "google.protobuf.internal", false);
-  } else {
-    return desc->full_name();
+    constexpr absl::string_view replace = "google.protobuf";
+    size_t index = full_name.find(replace);
+    if (index != std::string::npos) {
+      return absl::StrCat(full_name.substr(0, index),
+                          "google.protobuf.internal",
+                          full_name.substr(index + replace.size()));
+    }
   }
-}
-
-template <typename DescriptorType>
-std::string ClassNamePrefix(const std::string& classname,
-                            const DescriptorType* desc) {
-  const std::string& prefix = (desc->file()->options()).php_class_prefix();
-  if (!prefix.empty()) {
-    return prefix;
-  }
-
-  return ReservedNamePrefix(classname, desc->file());
-}
-
-template <typename DescriptorType>
-std::string GeneratedClassNameImpl(const DescriptorType* desc) {
-  std::string classname = ClassNamePrefix(desc->name(), desc) + desc->name();
-  const Descriptor* containing = desc->containing_type();
-  while (containing != NULL) {
-    classname = ClassNamePrefix(containing->name(), desc) + containing->name()
-       + '\\' + classname;
-    containing = containing->containing_type();
-  }
-  return classname;
-}
-
-std::string GeneratedClassNameImpl(const ServiceDescriptor* desc) {
-  std::string classname = desc->name();
-  return ClassNamePrefix(classname, desc) + classname;
+  return std::string(full_name);
 }
 
 template <typename DescriptorType>
@@ -196,31 +139,13 @@ std::string LegacyGeneratedClassName(const DescriptorType* desc) {
   return ClassNamePrefix(classname, desc) + classname;
 }
 
-std::string ClassNamePrefix(const std::string& classname) {
-  std::string lower = classname;
-  std::transform(lower.begin(), lower.end(), lower.begin(), ::tolower);
-
-  for (int i = 0; i < kReservedNamesSize; i++) {
-    if (lower == kReservedNames[i]) {
-      return "PB";
-    }
-  }
-
-  return "";
-}
-
-std::string ConstantNamePrefix(const std::string& classname) {
+std::string ConstantNamePrefix(absl::string_view classname) {
   bool is_reserved = false;
 
-  std::string lower = classname;
+  std::string lower(classname);
   std::transform(lower.begin(), lower.end(), lower.begin(), ::tolower);
 
-  for (int i = 0; i < kReservedNamesSize; i++) {
-    if (lower == kReservedNames[i]) {
-      is_reserved = true;
-      break;
-    }
-  }
+  is_reserved = IsReservedName(lower);
 
   for (int i = 0; i < kValidConstantNamesSize; i++) {
     if (lower == kValidConstantNames[i]) {
@@ -240,9 +165,9 @@ template <typename DescriptorType>
 std::string RootPhpNamespace(const DescriptorType* desc,
                              const Options& options) {
   if (desc->file()->options().has_php_namespace()) {
-    const std::string& php_namespace = desc->file()->options().php_namespace();
+    absl::string_view php_namespace = desc->file()->options().php_namespace();
     if (!php_namespace.empty()) {
-      return php_namespace;
+      return std::string(php_namespace);
     }
     return "";
   }
@@ -255,10 +180,10 @@ std::string RootPhpNamespace(const DescriptorType* desc,
 
 template <typename DescriptorType>
 std::string FullClassName(const DescriptorType* desc, const Options& options) {
-  std::string classname = GeneratedClassNameImpl(desc);
+  std::string classname = GeneratedClassName(desc);
   std::string php_namespace = RootPhpNamespace(desc, options);
   if (!php_namespace.empty()) {
-    return php_namespace + "\\" + classname;
+    return absl::StrCat(php_namespace, "\\", classname);
   }
   return classname;
 }
@@ -276,14 +201,19 @@ std::string LegacyFullClassName(const DescriptorType* desc,
   std::string classname = LegacyGeneratedClassName(desc);
   std::string php_namespace = RootPhpNamespace(desc, options);
   if (!php_namespace.empty()) {
-    return php_namespace + "\\" + classname;
+    return absl::StrCat(php_namespace, "\\", classname);
   }
   return classname;
 }
 
-std::string PhpName(const std::string& full_name, const Options& options) {
+std::string PhpNamePrefix(absl::string_view classname) {
+  if (IsReservedName(classname)) return "PB";
+  return "";
+}
+
+std::string PhpName(absl::string_view full_name, const Options& options) {
   if (options.is_descriptor) {
-    return kDescriptorPackageName;
+    return std::string(kDescriptorPackageName);
   }
 
   std::string segment;
@@ -294,7 +224,7 @@ std::string PhpName(const std::string& full_name, const Options& options) {
       segment += full_name[i] + ('A' - 'a');
       cap_next_letter = false;
     } else if (full_name[i] == '.') {
-      result += ClassNamePrefix(segment) + segment + '\\';
+      result += PhpNamePrefix(segment) + segment + '\\';
       segment = "";
       cap_next_letter = true;
     } else {
@@ -302,7 +232,7 @@ std::string PhpName(const std::string& full_name, const Options& options) {
       cap_next_letter = false;
     }
   }
-  result += ClassNamePrefix(segment) + segment;
+  result += PhpNamePrefix(segment) + segment;
   return result;
 }
 
@@ -332,51 +262,51 @@ std::string DefaultForField(const FieldDescriptor* field) {
 
 std::string GeneratedMetadataFileName(const FileDescriptor* file,
                                       const Options& options) {
-  const std::string& proto_file = file->name();
+  absl::string_view proto_file = file->name();
   int start_index = 0;
-  int first_index = proto_file.find_first_of("/", start_index);
+  int first_index = proto_file.find_first_of('/', start_index);
   std::string result = "";
   std::string segment = "";
 
   if (proto_file == kEmptyFile) {
-    return kEmptyMetadataFile;
+    return std::string(kEmptyMetadataFile);
   }
   if (options.is_descriptor) {
-    return kDescriptorMetadataFile;
+    return std::string(kDescriptorMetadataFile);
   }
 
   // Append directory name.
-  std::string file_no_suffix;
-  int lastindex = proto_file.find_last_of(".");
+  absl::string_view file_no_suffix;
+  int lastindex = proto_file.find_last_of('.');
   if (proto_file == kEmptyFile) {
-    return kEmptyMetadataFile;
+    return std::string(kEmptyMetadataFile);
   } else {
     file_no_suffix = proto_file.substr(0, lastindex);
   }
 
   if (file->options().has_php_metadata_namespace()) {
-    const std::string& php_metadata_namespace =
+    absl::string_view php_metadata_namespace =
         file->options().php_metadata_namespace();
     if (!php_metadata_namespace.empty() && php_metadata_namespace != "\\") {
-      result += php_metadata_namespace;
+      absl::StrAppend(&result, php_metadata_namespace);
       std::replace(result.begin(), result.end(), '\\', '/');
       if (result.at(result.size() - 1) != '/') {
-        result += "/";
+        absl::StrAppend(&result, "/");
       }
     }
   } else {
-    result += "GPBMetadata/";
+    absl::StrAppend(&result, "GPBMetadata/");
     while (first_index != std::string::npos) {
       segment = UnderscoresToCamelCase(
           file_no_suffix.substr(start_index, first_index - start_index), true);
-      result += ReservedNamePrefix(segment, file) + segment + "/";
+      absl::StrAppend(&result, ReservedNamePrefix(segment, file), segment, "/");
       start_index = first_index + 1;
-      first_index = file_no_suffix.find_first_of("/", start_index);
+      first_index = file_no_suffix.find_first_of('/', start_index);
     }
   }
 
   // Append file name.
-  int file_name_start = file_no_suffix.find_last_of("/");
+  int file_name_start = file_no_suffix.find_last_of('/');
   if (file_name_start == std::string::npos) {
     file_name_start = 0;
   } else {
@@ -385,7 +315,8 @@ std::string GeneratedMetadataFileName(const FileDescriptor* file,
   segment = UnderscoresToCamelCase(
       file_no_suffix.substr(file_name_start, first_index - file_name_start), true);
 
-  return result + ReservedNamePrefix(segment, file) + segment + ".php";
+  return absl::StrCat(result, ReservedNamePrefix(segment, file), segment,
+                      ".php");
 }
 
 std::string GeneratedMetadataFileName(const FileDescriptor* file,
@@ -404,7 +335,7 @@ std::string GeneratedClassFileName(const DescriptorType* desc,
       result[i] = '/';
     }
   }
-  return result + ".php";
+  return absl::StrCat(result, ".php");
 }
 
 template <typename DescriptorType>
@@ -417,7 +348,7 @@ std::string LegacyGeneratedClassFileName(const DescriptorType* desc,
       result[i] = '/';
     }
   }
-  return result + ".php";
+  return absl::StrCat(result, ".php");
 }
 
 template <typename DescriptorType>
@@ -429,21 +360,21 @@ std::string LegacyReadOnlyGeneratedClassFileName(std::string php_namespace,
         php_namespace[i] = '/';
       }
     }
-    return php_namespace + "/" + desc->name() + ".php";
+    return absl::StrCat(php_namespace, "/", desc->name(), ".php");
   }
 
-  return desc->name() + ".php";
+  return absl::StrCat(desc->name(), ".php");
 }
 
 std::string GeneratedServiceFileName(const ServiceDescriptor* service,
                                      const Options& options) {
-  std::string result = FullClassName(service, options) + "Interface";
+  std::string result = FullClassName(service, options);
   for (int i = 0; i < result.size(); i++) {
     if (result[i] == '\\') {
       result[i] = '/';
     }
   }
-  return result + ".php";
+  return absl::StrCat(result, "Interface", ".php");
 }
 
 std::string IntToString(int32_t value) {
@@ -495,7 +426,7 @@ std::string PhpSetterTypeName(const FieldDescriptor* field,
       type = "string";
       break;
     case FieldDescriptor::TYPE_MESSAGE:
-      type = "\\" + FullClassName(field->message_type(), options);
+      type = absl::StrCat("\\", FullClassName(field->message_type(), options));
       break;
     case FieldDescriptor::TYPE_GROUP:
       return "null";
@@ -503,11 +434,12 @@ std::string PhpSetterTypeName(const FieldDescriptor* field,
   }
   if (field->is_repeated()) {
     // accommodate for edge case with multiple types.
-    size_t start_pos = type.find("|");
+    size_t start_pos = type.find('|');
     if (start_pos != std::string::npos) {
       type.replace(start_pos, 1, ">|array<");
     }
-    type = "array<" + type + ">|\\Google\\Protobuf\\Internal\\RepeatedField";
+    type = absl::StrCat("array<", type,
+                        ">|\\Google\\Protobuf\\Internal\\RepeatedField");
   }
   return type;
 }
@@ -545,7 +477,7 @@ std::string PhpGetterTypeName(const FieldDescriptor* field,
     case FieldDescriptor::TYPE_STRING:
     case FieldDescriptor::TYPE_BYTES: return "string";
     case FieldDescriptor::TYPE_MESSAGE:
-      return "\\" + FullClassName(field->message_type(), options);
+      return absl::StrCat("\\", FullClassName(field->message_type(), options));
     case FieldDescriptor::TYPE_GROUP: return "null";
     default: assert(false); return "";
   }
@@ -561,13 +493,14 @@ std::string PhpGetterTypeName(const FieldDescriptor* field,
 std::string EnumOrMessageSuffix(const FieldDescriptor* field,
                                 const Options& options) {
   if (field->cpp_type() == FieldDescriptor::CPPTYPE_MESSAGE) {
-    return ", '" +
-           DescriptorFullName(field->message_type(), options.is_descriptor) +
-           "'";
+    return absl::StrCat(
+        ", '", DescriptorFullName(field->message_type(), options.is_descriptor),
+        "'");
   }
   if (field->cpp_type() == FieldDescriptor::CPPTYPE_ENUM) {
-    return ", '" +
-           DescriptorFullName(field->enum_type(), options.is_descriptor) + "'";
+    return absl::StrCat(
+        ", '", DescriptorFullName(field->enum_type(), options.is_descriptor),
+        "'");
   }
   return "";
 }
@@ -581,7 +514,7 @@ std::string EnumOrMessageSuffix(const FieldDescriptor* field,
 
 // Converts a name to camel-case. If cap_first_letter is true, capitalize the
 // first letter.
-std::string UnderscoresToCamelCase(const std::string& name,
+std::string UnderscoresToCamelCase(absl::string_view name,
                                    bool cap_first_letter) {
   std::string result;
   for (int i = 0; i < name.size(); i++) {
@@ -662,8 +595,11 @@ void GenerateFieldAccessor(const FieldDescriptor* field, const Options& options,
   GenerateFieldDocComment(printer, field, options, kFieldGetter);
 
   // deprecation
-  std::string deprecation_trigger = (field->options().deprecated()) ? "@trigger_error('" +
-      field->name() + " is deprecated.', E_USER_DEPRECATED);\n        " : "";
+  std::string deprecation_trigger =
+      (field->options().deprecated())
+          ? absl::StrCat("@trigger_error('", field->name(),
+                         " is deprecated.', E_USER_DEPRECATED);\n        ")
+          : "";
 
   // Emit getter.
   if (oneof != NULL) {
@@ -763,18 +699,16 @@ void GenerateFieldAccessor(const FieldDescriptor* field, const Options& options,
         "$arr = GPBUtil::checkMapField($var, "
         "\\Google\\Protobuf\\Internal\\GPBType::^key_type^, "
         "\\Google\\Protobuf\\Internal\\GPBType::^value_type^",
-        "key_type", ToUpper(key->type_name()),
-        "value_type", ToUpper(value->type_name()));
+        "key_type", absl::AsciiStrToUpper(key->type_name()),
+        "value_type", absl::AsciiStrToUpper(value->type_name()));
     if (value->cpp_type() == FieldDescriptor::CPPTYPE_MESSAGE) {
-      printer->Print(
-          ", \\^class_name^);\n",
-          "class_name",
-          FullClassName(value->message_type(), options) + "::class");
+      printer->Print(", \\^class_name^);\n", "class_name",
+                     absl::StrCat(FullClassName(value->message_type(), options),
+                                  "::class"));
     } else if (value->cpp_type() == FieldDescriptor::CPPTYPE_ENUM) {
       printer->Print(
-          ", \\^class_name^);\n",
-          "class_name",
-          FullClassName(value->enum_type(), options) + "::class");
+          ", \\^class_name^);\n", "class_name",
+          absl::StrCat(FullClassName(value->enum_type(), options), "::class"));
     } else {
       printer->Print(");\n");
     }
@@ -782,17 +716,15 @@ void GenerateFieldAccessor(const FieldDescriptor* field, const Options& options,
     printer->Print(
         "$arr = GPBUtil::checkRepeatedField($var, "
         "\\Google\\Protobuf\\Internal\\GPBType::^type^",
-        "type", ToUpper(field->type_name()));
+        "type", absl::AsciiStrToUpper(field->type_name()));
     if (field->cpp_type() == FieldDescriptor::CPPTYPE_MESSAGE) {
-      printer->Print(
-          ", \\^class_name^);\n",
-          "class_name",
-          FullClassName(field->message_type(), options) + "::class");
+      printer->Print(", \\^class_name^);\n", "class_name",
+                     absl::StrCat(FullClassName(field->message_type(), options),
+                                  "::class"));
     } else if (field->cpp_type() == FieldDescriptor::CPPTYPE_ENUM) {
       printer->Print(
-          ", \\^class_name^);\n",
-          "class_name",
-          FullClassName(field->enum_type(), options) + "::class");
+          ", \\^class_name^);\n", "class_name",
+          absl::StrCat(FullClassName(field->enum_type(), options), "::class"));
     } else {
       printer->Print(");\n");
     }
@@ -882,16 +814,16 @@ void GenerateServiceMethod(const MethodDescriptor* method,
   );
 }
 
-void GenerateMessageToPool(const std::string& name_prefix,
+void GenerateMessageToPool(absl::string_view name_prefix,
                            const Descriptor* message, io::Printer* printer) {
   // Don't generate MapEntry messages -- we use the PHP extension's native
   // support for map fields instead.
   if (message->options().map_entry()) {
     return;
   }
-  std::string class_name =
-      (name_prefix.empty() ? "" : name_prefix + "\\") +
-      ReservedNamePrefix(message->name(), message->file()) + message->name();
+  std::string class_name = absl::StrCat(
+      name_prefix.empty() ? "" : absl::StrCat(name_prefix, "\\"),
+      ReservedNamePrefix(message->name(), message->file()), message->name());
 
   printer->Print(
       "$pool->addMessage('^message^', "
@@ -912,9 +844,9 @@ void GenerateMessageToPool(const std::string& name_prefix,
           "->map('^field^', \\Google\\Protobuf\\Internal\\GPBType::^key^, "
           "\\Google\\Protobuf\\Internal\\GPBType::^value^, ^number^^other^)\n",
           "field", field->name(),
-          "key", ToUpper(key->type_name()),
-          "value", ToUpper(val->type_name()),
-          "number", StrCat(field->number()),
+          "key", absl::AsciiStrToUpper(key->type_name()),
+          "value", absl::AsciiStrToUpper(val->type_name()),
+          "number", absl::StrCat(field->number()),
           "other", EnumOrMessageSuffix(val, true));
     } else if (!field->real_containing_oneof()) {
       printer->Print(
@@ -922,8 +854,8 @@ void GenerateMessageToPool(const std::string& name_prefix,
           "\\Google\\Protobuf\\Internal\\GPBType::^type^, ^number^^other^)\n",
           "field", field->name(),
           "label", LabelForField(field),
-          "type", ToUpper(field->type_name()),
-          "number", StrCat(field->number()),
+          "type", absl::AsciiStrToUpper(field->type_name()),
+          "number", absl::StrCat(field->number()),
           "other", EnumOrMessageSuffix(field, true));
     }
   }
@@ -940,8 +872,8 @@ void GenerateMessageToPool(const std::string& name_prefix,
           "->value('^field^', "
           "\\Google\\Protobuf\\Internal\\GPBType::^type^, ^number^^other^)\n",
           "field", field->name(),
-          "type", ToUpper(field->type_name()),
-          "number", StrCat(field->number()),
+          "type", absl::AsciiStrToUpper(field->type_name()),
+          "number", absl::StrCat(field->number()),
           "other", EnumOrMessageSuffix(field, true));
     }
     printer->Print("->finish()\n");
@@ -993,7 +925,7 @@ void GenerateAddFileToPool(const FileDescriptor* file, const Options& options,
           "$pool->finish();\n");
     } else {
       for (int i = 0; i < file->dependency_count(); i++) {
-        const std::string& name = file->dependency(i)->name();
+        absl::string_view name = file->dependency(i)->name();
         // Currently, descriptor.proto is not ready for external usage. Skip to
         // import it for now, so that its dependencies can still work as long as
         // they don't use protos defined in descriptor.proto.
@@ -1068,9 +1000,10 @@ void GenerateAddFileToPool(const FileDescriptor* file, const Options& options,
 
 static void AnalyzeDependencyForFile(
     const FileDescriptor* file,
-    std::set<const FileDescriptor*>* nodes_without_dependency,
-    std::map<const FileDescriptor*, std::set<const FileDescriptor*>>* deps,
-    std::map<const FileDescriptor*, int>* dependency_count) {
+    absl::flat_hash_set<const FileDescriptor*>* nodes_without_dependency,
+    absl::flat_hash_map<const FileDescriptor*,
+                        absl::flat_hash_set<const FileDescriptor*>>* deps,
+    absl::flat_hash_map<const FileDescriptor*, int>* dependency_count) {
   int count = file->dependency_count();
   for (int i = 0; i < file->dependency_count(); i++) {
       const FileDescriptor* dependency = file->dependency(i);
@@ -1090,7 +1023,7 @@ static void AnalyzeDependencyForFile(
         continue;
       }
       if (deps->find(dependency) == deps->end()) {
-        (*deps)[dependency] = std::set<const FileDescriptor*>();
+        (*deps)[dependency] = {};
       }
       (*deps)[dependency].insert(file);
       AnalyzeDependencyForFile(
@@ -1106,7 +1039,7 @@ static bool NeedsUnwrapping(const FileDescriptor* file,
     has_aggregate_metadata_prefix = true;
   } else {
     for (const auto& prefix : options.aggregate_metadata_prefixes) {
-      if (HasPrefixString(file->package(), prefix)) {
+      if (absl::StartsWith(file->package(), prefix)) {
         has_aggregate_metadata_prefix = true;
         break;
       }
@@ -1126,9 +1059,11 @@ void GenerateAddFilesToPool(const FileDescriptor* file, const Options& options,
       "}\n");
 
   // Sort files according to dependency
-  std::map<const FileDescriptor*, std::set<const FileDescriptor*>> deps;
-  std::map<const FileDescriptor*, int> dependency_count;
-  std::set<const FileDescriptor*> nodes_without_dependency;
+  absl::flat_hash_map<const FileDescriptor*,
+                      absl::flat_hash_set<const FileDescriptor*>>
+      deps;
+  absl::flat_hash_map<const FileDescriptor*, int> dependency_count;
+  absl::flat_hash_set<const FileDescriptor*> nodes_without_dependency;
   FileDescriptorSet sorted_file_set;
 
   AnalyzeDependencyForFile(
@@ -1234,10 +1169,10 @@ void GenerateHead(const FileDescriptor* file, io::Printer* printer) {
     "filename", file->name());
 }
 
-std::string FilenameToClassname(const std::string& filename) {
-  int lastindex = filename.find_last_of(".");
-  std::string result = filename.substr(0, lastindex);
-  for (int i = 0; i < result.size(); i++) {
+std::string FilenameToClassname(absl::string_view filename) {
+  size_t lastindex = filename.find_last_of('.');
+  std::string result(filename.substr(0, lastindex));
+  for (size_t i = 0; i < result.size(); i++) {
     if (result[i] == '/') {
       result[i] = '\\';
     }
@@ -1255,7 +1190,7 @@ void GenerateMetadataFile(const FileDescriptor* file, const Options& options,
   GenerateHead(file, &printer);
 
   std::string fullname = FilenameToClassname(filename);
-  int lastindex = fullname.find_last_of("\\");
+  int lastindex = fullname.find_last_of('\\');
 
   if (lastindex != std::string::npos) {
     printer.Print(
@@ -1310,7 +1245,7 @@ void LegacyGenerateClassFile(const FileDescriptor* file,
   Outdent(&printer);
   printer.Print("}\n");
   printer.Print("class_exists(^new^::class);\n",
-      "new", GeneratedClassNameImpl(desc));
+      "new", GeneratedClassName(desc));
   printer.Print("@trigger_error('^old^ is deprecated and will be removed in "
       "the next major release. Use ^fullname^ instead', E_USER_DEPRECATED);\n\n",
       "old", LegacyFullClassName(desc, options),
@@ -1367,7 +1302,7 @@ void GenerateEnumFile(const FileDescriptor* file, const EnumDescriptor* en,
   GenerateHead(file, &printer);
 
   std::string fullname = FilenameToClassname(filename);
-  int lastindex = fullname.find_last_of("\\");
+  int lastindex = fullname.find_last_of('\\');
 
   if (lastindex != std::string::npos) {
     printer.Print(
@@ -1467,7 +1402,7 @@ void GenerateEnumFile(const FileDescriptor* file, const EnumDescriptor* en,
   Outdent(&printer);
   printer.Print("}\n\n");
 
-  // write legacy file for backwards compatibility with nested messages and enums
+  // write legacy alias for backwards compatibility with nested messages and enums
   if (en->containing_type() != NULL) {
     printer.Print(
         "// Adding a class alias for backwards compatibility with the previous class name.\n");
@@ -1475,7 +1410,6 @@ void GenerateEnumFile(const FileDescriptor* file, const EnumDescriptor* en,
         "class_alias(^new^::class, \\^old^::class);\n\n",
         "new", fullname,
         "old", LegacyFullClassName(en, options));
-    LegacyGenerateClassFile(file, en, options, generator_context);
   }
 
   // Write legacy file for backwards compatibility with "readonly" keywword
@@ -1509,7 +1443,7 @@ void GenerateMessageFile(const FileDescriptor* file, const Descriptor* message,
   GenerateHead(file, &printer);
 
   std::string fullname = FilenameToClassname(filename);
-  int lastindex = fullname.find_last_of("\\");
+  int lastindex = fullname.find_last_of('\\');
 
   if (lastindex != std::string::npos) {
     printer.Print(
@@ -1595,7 +1529,7 @@ void GenerateMessageFile(const FileDescriptor* file, const Descriptor* message,
   Outdent(&printer);
   printer.Print("}\n\n");
 
-  // write legacy file for backwards compatibility with nested messages and enums
+  // write legacy alias for backwards compatibility with nested messages and enums
   if (message->containing_type() != NULL) {
     printer.Print(
         "// Adding a class alias for backwards compatibility with the previous class name.\n");
@@ -1603,7 +1537,6 @@ void GenerateMessageFile(const FileDescriptor* file, const Descriptor* message,
         "class_alias(^new^::class, \\^old^::class);\n\n",
         "new", fullname,
         "old", LegacyFullClassName(message, options));
-    LegacyGenerateClassFile(file, message, options, generator_context);
   }
 
   // Write legacy file for backwards compatibility with "readonly" keywword
@@ -1640,7 +1573,7 @@ void GenerateServiceFile(
   GenerateHead(file, &printer);
 
   std::string fullname = FilenameToClassname(filename);
-  int lastindex = fullname.find_last_of("\\");
+  int lastindex = fullname.find_last_of('\\');
 
   if (!file->options().php_namespace().empty() ||
       (!file->options().has_php_namespace() && !file->package().empty()) ||
@@ -1694,7 +1627,7 @@ void GenerateFile(const FileDescriptor* file, const Options& options,
   }
 }
 
-static std::string EscapePhpdoc(const std::string& input) {
+static std::string EscapePhpdoc(absl::string_view input) {
   std::string result;
   result.reserve(input.size() * 2);
 
@@ -1751,7 +1684,8 @@ static void GenerateDocCommentBodyForLocation(
     // HTML-escape them so that they don't accidentally close the doc comment.
     comments = EscapePhpdoc(comments);
 
-    std::vector<std::string> lines = Split(comments, "\n", true);
+    std::vector<absl::string_view> lines =
+        absl::StrSplit(comments, '\n', absl::SkipEmpty());
     while (!lines.empty() && lines.back().empty()) {
       lines.pop_back();
     }
@@ -1782,8 +1716,8 @@ static void GenerateDocCommentBody(
   }
 }
 
-static std::string FirstLineOf(const std::string& value) {
-  std::string result = value;
+static std::string FirstLineOf(absl::string_view value) {
+  std::string result(value);
 
   std::string::size_type pos = result.find_first_of('\n');
   if (pos != std::string::npos) {
@@ -1950,17 +1884,13 @@ void GenerateServiceMethodDocComment(io::Printer* printer,
 }
 
 std::string FilenameCName(const FileDescriptor* file) {
-  std::string c_name = file->name();
-  c_name = StringReplace(c_name, ".", "_", true);
-  c_name = StringReplace(c_name, "/", "_", true);
-  return c_name;
+  return absl::StrReplaceAll(file->name(), {{".", "_"}, {"/", "_"}});
 }
 
 void GenerateCEnum(const EnumDescriptor* desc, io::Printer* printer) {
-  std::string c_name = desc->full_name();
-  c_name = StringReplace(c_name, ".", "_", true);
-  std::string php_name = FullClassName(desc, Options());
-  php_name = StringReplace(php_name, "\\", "\\\\", true);
+  std::string c_name = absl::StrReplaceAll(desc->full_name(), {{".", "_"}});
+  std::string php_name =
+      absl::StrReplaceAll(FullClassName(desc, Options()), {{"\\", "\\\\"}});
   printer->Print(
       "/* $c_name$ */\n"
       "\n"
@@ -2043,10 +1973,9 @@ void GenerateCEnum(const EnumDescriptor* desc, io::Printer* printer) {
 }
 
 void GenerateCMessage(const Descriptor* message, io::Printer* printer) {
-  std::string c_name = message->full_name();
-  c_name = StringReplace(c_name, ".", "_", true);
-  std::string php_name = FullClassName(message, Options());
-  php_name = StringReplace(php_name, "\\", "\\\\", true);
+  std::string c_name = absl::StrReplaceAll(message->full_name(), {{".", "_"}});
+  std::string php_name =
+      absl::StrReplaceAll(FullClassName(message, Options()), {{"\\", "\\\\"}});
   printer->Print(
       "/* $c_name$ */\n"
       "\n"
@@ -2196,8 +2125,7 @@ void GenerateCMessage(const Descriptor* message, io::Printer* printer) {
 }
 
 void GenerateEnumCInit(const EnumDescriptor* desc, io::Printer* printer) {
-  std::string c_name = desc->full_name();
-  c_name = StringReplace(c_name, ".", "_", true);
+  std::string c_name = absl::StrReplaceAll(desc->full_name(), {{".", "_"}});
 
   printer->Print(
       "  $c_name$_ModuleInit();\n",
@@ -2205,8 +2133,7 @@ void GenerateEnumCInit(const EnumDescriptor* desc, io::Printer* printer) {
 }
 
 void GenerateCInit(const Descriptor* message, io::Printer* printer) {
-  std::string c_name = message->full_name();
-  c_name = StringReplace(c_name, ".", "_", true);
+  std::string c_name = absl::StrReplaceAll(message->full_name(), {{".", "_"}});
 
   printer->Print(
       "  $c_name$_ModuleInit();\n",
@@ -2248,8 +2175,9 @@ void GenerateCWellKnownTypes(const std::vector<const FileDescriptor*>& files,
     std::string metadata_filename = GeneratedMetadataFileName(file, Options());
     std::string metadata_classname = FilenameToClassname(metadata_filename);
     std::string metadata_c_name =
-        StringReplace(metadata_classname, "\\", "_", true);
-    metadata_classname = StringReplace(metadata_classname, "\\", "\\\\", true);
+        absl::StrReplaceAll(metadata_classname, {{"\\", "_"}});
+    metadata_classname =
+        absl::StrReplaceAll(metadata_classname, {{"\\", "\\\\"}});
     FileDescriptorProto file_proto;
     file->CopyTo(&file_proto);
     std::string serialized;
@@ -2267,7 +2195,7 @@ void GenerateCWellKnownTypes(const std::vector<const FileDescriptor*>& files,
 
     for (size_t i = 0; i < serialized.size();) {
       for (size_t j = 0; j < 25 && i < serialized.size(); ++i, ++j) {
-        printer.Print("'$ch$', ", "ch", CEscape(serialized.substr(i, 1)));
+        printer.Print("'$ch$', ", "ch", absl::CEscape(serialized.substr(i, 1)));
       }
       printer.Print("\n");
     }
@@ -2330,7 +2258,7 @@ void GenerateCWellKnownTypes(const std::vector<const FileDescriptor*>& files,
     std::string metadata_filename = GeneratedMetadataFileName(file, Options());
     std::string metadata_classname = FilenameToClassname(metadata_filename);
     std::string metadata_c_name =
-        StringReplace(metadata_classname, "\\", "_", true);
+        absl::StrReplaceAll(metadata_classname, {{"\\", "_"}});
     printer.Print(
         "  $metadata_c_name$_ModuleInit();\n",
         "metadata_c_name", metadata_c_name);
@@ -2348,18 +2276,6 @@ void GenerateCWellKnownTypes(const std::vector<const FileDescriptor*>& files,
 
 }  // namespace
 
-std::string GeneratedClassName(const Descriptor* desc) {
-  return GeneratedClassNameImpl(desc);
-}
-
-std::string GeneratedClassName(const EnumDescriptor* desc) {
-  return GeneratedClassNameImpl(desc);
-}
-
-std::string GeneratedClassName(const ServiceDescriptor* desc) {
-  return GeneratedClassNameImpl(desc);
-}
-
 bool Generator::Generate(const FileDescriptor* file,
                          const std::string& parameter,
                          GeneratorContext* generator_context,
@@ -2394,20 +2310,20 @@ bool Generator::GenerateAll(const std::vector<const FileDescriptor*>& files,
                             std::string* error) const {
   Options options;
 
-  for (const auto& option : Split(parameter, ",", true)) {
-    const std::vector<std::string> option_pair = Split(option, "=", true);
-    if (HasPrefixString(option_pair[0], "aggregate_metadata")) {
+  for (const auto& option : absl::StrSplit(parameter, ",", absl::SkipEmpty())) {
+    const std::vector<std::string> option_pair = absl::StrSplit(option, "=", absl::SkipEmpty());
+    if (absl::StartsWith(option_pair[0], "aggregate_metadata")) {
       options.aggregate_metadata = true;
-      for (const auto& prefix : Split(option_pair[1], "#", false)) {
+      for (const auto& prefix : absl::StrSplit(option_pair[1], "#", absl::AllowEmpty())) {
         options.aggregate_metadata_prefixes.emplace(prefix);
-        GOOGLE_LOG(INFO) << prefix;
+        ABSL_LOG(INFO) << prefix;
       }
     } else if (option_pair[0] == "internal") {
       options.is_descriptor = true;
     } else if (option_pair[0] == "internal_generate_c_wkt") {
       GenerateCWellKnownTypes(files, generator_context);
     } else {
-      GOOGLE_LOG(FATAL) << "Unknown codegen option: " << option_pair[0];
+      ABSL_LOG(FATAL) << "Unknown codegen option: " << option_pair[0];
     }
   }
 
