@@ -30,10 +30,11 @@
 
 #include <cstddef>
 
-#include <google/protobuf/generated_message_tctable_impl.h>
-#include <google/protobuf/wire_format_lite.h>
+#include "google/protobuf/generated_message_tctable_impl.h"
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
+#include "absl/types/optional.h"
+#include "google/protobuf/wire_format_lite.h"
 
 namespace google {
 namespace protobuf {
@@ -43,9 +44,238 @@ namespace {
 
 using ::testing::Eq;
 using ::testing::Not;
+using ::testing::Optional;
+
+// The fast parser's dispatch table Xors two bytes of incoming data with
+// the data in TcFieldData, so we reproduce that here:
+TcFieldData Xor2SerializedBytes(TcFieldData tfd, const char* ptr) {
+  uint64_t twobytes = 0xFF & ptr[0];
+  twobytes |= (0xFF & ptr[1]) << 8;
+  tfd.data ^= twobytes;
+  return tfd;
+}
+
+absl::optional<const char*> fallback_ptr_received;
+absl::optional<uint64_t> fallback_hasbits_received;
+absl::optional<uint64_t> fallback_tag_received;
+const char* FastParserGaveUp(::google::protobuf::MessageLite*, const char* ptr,
+                             ::google::protobuf::internal::ParseContext*,
+                             ::google::protobuf::internal::TcFieldData data,
+                             const ::google::protobuf::internal::TcParseTableBase*,
+                             uint64_t hasbits) {
+  fallback_ptr_received = ptr;
+  fallback_hasbits_received = hasbits;
+  fallback_tag_received = data.tag();
+  return nullptr;
+}
+
+// To test that we aren't storing too much data, we set up a fake message area
+// and fill all its bytes with kDND.
+constexpr char kDND = 0x5A;  // "Do Not Disturb"
+
+// To retrieve data and see if it matches what we expect, we have this routine
+// which simultaneously reads the data we want, and sets it back to what it was
+// before the test, that is, to kDND.  This makes it easier to test at the end
+// that all the original data is undisturbed.
+template <typename T>
+T ReadAndReset(char* p) {
+  T result;
+  memcpy(&result, p, sizeof(result));
+  memset(p, kDND, sizeof(result));
+  return result;
+}
+
+TEST(FastVarints, NameHere) {
+  constexpr uint8_t kHasBitsOffset = 4;
+  constexpr uint8_t kHasBitIndex = 0;
+  constexpr uint8_t kFieldOffset = 24;
+
+  // clang-format on
+  const TcParseTable<0, 1, 0, 0, 2> parse_table = {
+      {
+          kHasBitsOffset,  //
+          0, 0, 0,         // no _extensions_
+          1, 0,            // max_field_number, fast_idx_mask
+          offsetof(decltype(parse_table), field_lookup_table),
+          0xFFFFFFFF - 1,  // skipmap
+          offsetof(decltype(parse_table), field_entries),
+          1,                                             // num_field_entries
+          0,                                             // num_aux_entries
+          offsetof(decltype(parse_table), field_names),  // no aux_entries
+          nullptr,                                       // default instance
+          FastParserGaveUp,                              // fallback
+      },
+      // Fast Table:
+      {{
+          // optional int32 field = 1;
+          {TcParser::SingularVarintNoZag1<::uint32_t, kFieldOffset,
+                                          kHasBitIndex>(),
+           {/* coded_tag= */ 8, kHasBitIndex, /* aux_idx= */ 0, kFieldOffset}},
+      }},
+      // Field Lookup Table:
+      {{65535, 65535}},
+      // Field Entries:
+      {{
+          // This is set to kFkNone to force MiniParse to call the fallback
+          {kFieldOffset, kHasBitsOffset + 0, 0, (field_layout::kFkNone)},
+      }},
+      // no aux_entries
+      {{}},
+  };
+  // clang-format on
+  uint8_t serialize_buffer[64];
+
+  for (int size : {8, 32, 64, -8, -32, -64}) {
+    SCOPED_TRACE(size);
+    auto next_i = [](uint64_t i) {
+      // if i + 1 is a power of two, return that.
+      // (This will also match when i == -1, but for this loop we know that will
+      // not happen.)
+      if ((i & (i + 1)) == 0) return i + 1;
+      // otherwise, i is already a power of two, so advance to one less than the
+      // next power of two.
+      return i + (i - 1);
+    };
+    for (uint64_t i = 0; i + 1 != 0; i = next_i(i)) {
+      SCOPED_TRACE(i);
+      enum OverlongKind { kNotOverlong, kOverlong, kOverlongWithDroppedBits };
+      for (OverlongKind overlong :
+           {kNotOverlong, kOverlong, kOverlongWithDroppedBits}) {
+        SCOPED_TRACE(overlong);
+        alignas(16) char fake_msg[64] = {
+            kDND, kDND, kDND, kDND, kDND, kDND, kDND, kDND,  //
+            kDND, kDND, kDND, kDND, kDND, kDND, kDND, kDND,  //
+            kDND, kDND, kDND, kDND, kDND, kDND, kDND, kDND,  //
+            kDND, kDND, kDND, kDND, kDND, kDND, kDND, kDND,  //
+            kDND, kDND, kDND, kDND, kDND, kDND, kDND, kDND,  //
+            kDND, kDND, kDND, kDND, kDND, kDND, kDND, kDND,  //
+            kDND, kDND, kDND, kDND, kDND, kDND, kDND, kDND,  //
+            kDND, kDND, kDND, kDND, kDND, kDND, kDND, kDND,  //
+        };
+        memset(&fake_msg[kHasBitsOffset], 0, sizeof(uint32_t));
+
+        auto serialize_ptr = WireFormatLite::WriteUInt64ToArray(
+            /* field_number= */ 1, i, serialize_buffer);
+
+        if (overlong == kOverlong || overlong == kOverlongWithDroppedBits) {
+          // 1 for the tag plus 10 for the value
+          while (serialize_ptr - serialize_buffer < 11) {
+            serialize_ptr[-1] |= 0x80;
+            *serialize_ptr++ = 0;
+          }
+          if (overlong == kOverlongWithDroppedBits) {
+            // For this one we add some unused bits to the last byte.
+            // They should be dropped. Bits 1-6 are dropped. Bit 0 is used and
+            // bit 7 is checked for continuation.
+            serialize_ptr[-1] |= 0b0111'1110;
+          }
+        }
+
+        absl::string_view serialized{
+            reinterpret_cast<char*>(&serialize_buffer[0]),
+            static_cast<size_t>(serialize_ptr - serialize_buffer)};
+
+        const char* ptr = nullptr;
+        const char* end_ptr = nullptr;
+        ParseContext ctx(io::CodedInputStream::GetDefaultRecursionLimit(),
+                         /* aliasing= */ false, &ptr, serialized);
+#if 0  // FOR_DEBUGGING
+      ABSL_LOG(ERROR) << "size=" << size << " i=" << i << " ptr points to "  //
+                      << +ptr[0] << "," << +ptr[1] << ","                    //
+                      << +ptr[2] << "," << +ptr[3] << ","                    //
+                      << +ptr[4] << "," << +ptr[5] << ","                    //
+                      << +ptr[6] << "," << +ptr[7] << ","                    //
+                      << +ptr[8] << "," << +ptr[9] << "," << +ptr[10] << "\n";
+#endif
+        TailCallParseFunc fn = nullptr;
+        switch (size) {
+          case 8:
+            fn = &TcParser::FastV8S1;
+            break;
+          case -8:
+            fn = &TcParser::FastTV8S1<kFieldOffset, kHasBitIndex>;
+            break;
+          case 32:
+            fn = &TcParser::FastV32S1;
+            break;
+          case -32:
+            fn = &TcParser::FastTV32S1<uint32_t, kFieldOffset, kHasBitIndex>;
+            break;
+          case 64:
+            fn = &TcParser::FastV64S1;
+            break;
+          case -64:
+            fn = &TcParser::FastTV64S1<uint64_t, kFieldOffset, kHasBitIndex>;
+            break;
+        }
+        fallback_ptr_received = absl::nullopt;
+        fallback_hasbits_received = absl::nullopt;
+        fallback_tag_received = absl::nullopt;
+        end_ptr = fn(reinterpret_cast<MessageLite*>(fake_msg), ptr, &ctx,
+                     Xor2SerializedBytes(parse_table.fast_entries[0].bits, ptr),
+                     &parse_table.header, /*hasbits=*/0);
+        switch (size) {
+          case -8:
+          case 8: {
+            if (end_ptr == nullptr) {
+              // If end_ptr is nullptr, that means the FastParser gave up and
+              // tried to pass control to MiniParse.... which is expected
+              // anytime we encounter something other than 0 or 1 encodings.
+              // (Since FastV8S1 is only used for `bool` fields.)
+              if (overlong == kNotOverlong) {
+                EXPECT_NE(i, true);
+                EXPECT_NE(i, false);
+              }
+              EXPECT_THAT(fallback_hasbits_received, Optional(0));
+              // Like the mini-parser functions, and unlike the fast-parser
+              // functions, the fallback receives a ptr already incremented past
+              // the tag, and receives the actual tag in the `data` parameter.
+              EXPECT_THAT(fallback_ptr_received, Optional(ptr + 1));
+              EXPECT_THAT(fallback_tag_received, Optional(0x7F & *ptr));
+              continue;
+            }
+            ASSERT_EQ(end_ptr - ptr, serialized.size());
+
+            auto actual_field = ReadAndReset<uint8_t>(&fake_msg[kFieldOffset]);
+            EXPECT_EQ(actual_field, static_cast<decltype(actual_field)>(i))  //
+                << " hex: " << absl::StrCat(absl::Hex(actual_field));
+          }; break;
+          case -32:
+          case 32: {
+            ASSERT_TRUE(end_ptr);
+            ASSERT_EQ(end_ptr - ptr, serialized.size());
+
+            auto actual_field = ReadAndReset<uint32_t>(&fake_msg[kFieldOffset]);
+            EXPECT_EQ(actual_field, static_cast<decltype(actual_field)>(i))  //
+                << " hex: " << absl::StrCat(absl::Hex(actual_field));
+          }; break;
+          case -64:
+          case 64: {
+            ASSERT_EQ(end_ptr - ptr, serialized.size());
+
+            auto actual_field = ReadAndReset<uint64_t>(&fake_msg[kFieldOffset]);
+            EXPECT_EQ(actual_field, static_cast<decltype(actual_field)>(i))  //
+                << " hex: " << absl::StrCat(absl::Hex(actual_field));
+          }; break;
+        }
+        EXPECT_TRUE(!fallback_ptr_received);
+        EXPECT_TRUE(!fallback_hasbits_received);
+        EXPECT_TRUE(!fallback_tag_received);
+        auto hasbits = ReadAndReset<uint32_t>(&fake_msg[kHasBitsOffset]);
+        EXPECT_EQ(hasbits, 1 << kHasBitIndex);
+
+        int offset = 0;
+        for (char ch : fake_msg) {
+          EXPECT_EQ(ch, kDND) << " corruption of message at offset " << offset;
+          ++offset;
+        }
+      }
+    }
+  }
+}
 
 MATCHER_P3(IsEntryForFieldNum, table, field_num, field_numbers_table,
-           StrCat(negation ? "isn't " : "",
+           absl::StrCat(negation ? "isn't " : "",
                         "the field entry for field number ", field_num)) {
   if (arg == nullptr) {
     *result_listener << "which is nullptr";
@@ -80,7 +310,7 @@ TEST(IsEntryForFieldNumTest, Matcher) {
       }};
   // clang-format on
   int table_field_numbers[] = {1, 2, 3};
-  table.field_lookup_table = {65535, 65535};
+  table.field_lookup_table = {{65535, 65535}};
 
   auto& entries = table.field_entries;
   EXPECT_THAT(&entries[0], IsEntryForFieldNum(&table, 1, table_field_numbers));
@@ -108,7 +338,7 @@ class FindFieldEntryTest : public ::testing::Test {
   // Calls the private `FieldName` function.
   template <size_t kFastTableSizeLog2, size_t kNumEntries, size_t kNumFieldAux,
             size_t kNameTableSize, size_t kFieldLookupTableSize>
-  static StringPiece FieldName(
+  static absl::string_view FieldName(
       const TcParseTable<kFastTableSizeLog2, kNumEntries, kNumFieldAux,
                          kNameTableSize, kFieldLookupTableSize>& table,
       const TcParseTableBase::FieldEntry* entry) {
@@ -118,7 +348,7 @@ class FindFieldEntryTest : public ::testing::Test {
   // Calls the private `MessageName` function.
   template <size_t kFastTableSizeLog2, size_t kNumEntries, size_t kNumFieldAux,
             size_t kNameTableSize, size_t kFieldLookupTableSize>
-  static StringPiece MessageName(
+  static absl::string_view MessageName(
       const TcParseTable<kFastTableSizeLog2, kNumEntries, kNumFieldAux,
                          kNameTableSize, kFieldLookupTableSize>& table) {
     return TcParser::MessageName(&table.header);
@@ -163,7 +393,7 @@ TEST_F(FindFieldEntryTest, SequentialFieldRange) {
                 IsEntryForFieldNum(&table, i, table_field_numbers));
   }
   for (int i : {0, 1, 6, 7, 110, 112, 500000000}) {
-    GOOGLE_LOG(WARNING) << "Field " << i;
+    ABSL_LOG(WARNING) << "Field " << i;
     EXPECT_THAT(FindFieldEntry(table, i), Eq(nullptr));
   }
 }
@@ -300,10 +530,10 @@ TEST_F(FindFieldEntryTest, OutOfRange) {
     EXPECT_THAT(entry,
                 IsEntryForFieldNum(&table, field_num, table_field_numbers));
 
-    StringPiece name = FieldName(table, entry);
+    absl::string_view name = FieldName(table, entry);
     EXPECT_EQ(name.length(), field_num);
     while (name[0] == '0') name.remove_prefix(1);  // strip leading zeores
-    EXPECT_EQ(name, StrCat(field_num));
+    EXPECT_EQ(name, absl::StrCat(field_num));
   }
   for (int field_num : {0, 4, 112, 500000000}) {
     EXPECT_THAT(FindFieldEntry(table, field_num), Eq(nullptr));
@@ -557,7 +787,7 @@ TEST_F(FindFieldEntryTest, BigMessage) {
   for (int field_num :
        {1, 12, 31, 42, 57, 68, 79, 90, 101, 119, 249, 402, 412}) {
     auto* entry = FindFieldEntry(test_all_types_table, field_num);
-    StringPiece name = FieldName(test_all_types_table, entry);
+    absl::string_view name = FieldName(test_all_types_table, entry);
     switch (field_num) {
       case 1:
         EXPECT_THAT(name, Eq("optional_int32"));
