@@ -1,32 +1,50 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
+// https://developers.google.com/protocol-buffers/
 //
-// Use of this source code is governed by a BSD-style
-// license that can be found in the LICENSE file or at
-// https://developers.google.com/open-source/licenses/bsd
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are
+// met:
+//
+//     * Redistributions of source code must retain the above copyright
+// notice, this list of conditions and the following disclaimer.
+//     * Redistributions in binary form must reproduce the above
+// copyright notice, this list of conditions and the following disclaimer
+// in the documentation and/or other materials provided with the
+// distribution.
+//     * Neither the name of Google Inc. nor the names of its
+// contributors may be used to endorse or promote products derived from
+// this software without specific prior written permission.
+//
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 // Author: kenton@google.com (Kenton Varda)
 //  Based on original Protocol Buffers design by
 //  Sanjay Ghemawat, Jeff Dean, and others.
 //
-// TODO:  Improve this unittest to bring it up to the standards of
+// TODO(kenton):  Improve this unittest to bring it up to the standards of
 //   other proto2 unittests.
 
 #include "google/protobuf/repeated_field.h"
 
 #include <algorithm>
-#include <cstddef>
-#include <cstdint>
 #include <cstdlib>
-#include <cstring>
-#include <functional>
 #include <iterator>
 #include <limits>
 #include <list>
 #include <sstream>
 #include <string>
 #include <type_traits>
-#include <utility>
 #include <vector>
 
 #include <gmock/gmock.h>
@@ -35,13 +53,6 @@
 #include "absl/numeric/bits.h"
 #include "absl/strings/cord.h"
 #include "absl/strings/str_cat.h"
-#include "absl/types/span.h"
-#include "google/protobuf/arena_test_util.h"
-#include "google/protobuf/internal_visibility_for_testing.h"
-#include "google/protobuf/io/coded_stream.h"
-#include "google/protobuf/io/zero_copy_stream_impl_lite.h"
-#include "google/protobuf/parse_context.h"
-#include "google/protobuf/repeated_ptr_field.h"
 #include "google/protobuf/unittest.pb.h"
 
 
@@ -53,7 +64,6 @@ namespace protobuf {
 namespace {
 
 using ::protobuf_unittest::TestAllTypes;
-using ::protobuf_unittest::TestMessageWithManyRepeatedPtrFields;
 using ::testing::AllOf;
 using ::testing::ElementsAre;
 using ::testing::Ge;
@@ -191,24 +201,34 @@ TEST(RepeatedField, ArenaAllocationSizesMatchExpectedValues) {
   CheckAllocationSizes<RepeatedField<uint64_t>>(false);
 }
 
-TEST(RepeatedField, NaturalGrowthOnArenasReuseBlocks) {
+template <typename Rep>
+void CheckNaturalGrowthOnArenasReuseBlocks(bool is_ptr) {
   Arena arena;
-  std::vector<RepeatedField<int>*> values;
+  std::vector<Rep*> values;
+  using T = typename Rep::value_type;
 
   static constexpr int kNumFields = 100;
   static constexpr int kNumElems = 1000;
   for (int i = 0; i < kNumFields; ++i) {
-    values.push_back(Arena::CreateMessage<RepeatedField<int>>(&arena));
+    values.push_back(Arena::CreateMessage<Rep>(&arena));
     auto& field = *values.back();
     for (int j = 0; j < kNumElems; ++j) {
-      field.Add(j);
+      field.Add(T{});
     }
   }
 
-  size_t expected = values.size() * values[0]->Capacity() * sizeof(int);
-  // Use a 2% slack for other overhead. If we were not reusing the blocks, the
-  // actual value would be ~2x the expected.
-  EXPECT_THAT(arena.SpaceUsed(), AllOf(Ge(expected), Le(1.02 * expected)));
+  size_t used_bytes_if_reusing =
+      values.size() * values[0]->Capacity() * (is_ptr ? sizeof(T*) : sizeof(T));
+  // Use a 2% slack for other overhead.
+  // If we were not reusing the blocks, the actual value would be ~2x the
+  // expected.
+  EXPECT_THAT(
+      arena.SpaceUsed() - (is_ptr ? sizeof(T) * kNumElems * kNumFields : 0),
+      AllOf(Ge(used_bytes_if_reusing), Le(1.02 * used_bytes_if_reusing)));
+}
+
+TEST(RepeatedField, NaturalGrowthOnArenasReuseBlocks) {
+  CheckNaturalGrowthOnArenasReuseBlocks<RepeatedField<int>>(false);
 }
 
 // Test swapping between various types of RepeatedFields.
@@ -691,73 +711,16 @@ TEST(RepeatedField, AddAndAssignRanges) {
   EXPECT_EQ(field.Get(7), 609250);
 }
 
-TEST(RepeatedField, CopyConstructIntegers) {
-  auto token = internal::InternalVisibilityForTesting{};
-  using RepeatedType = RepeatedField<int>;
-  RepeatedType original;
-  original.Add(1);
-  original.Add(2);
+TEST(RepeatedField, CopyConstruct) {
+  RepeatedField<int> source;
+  source.Add(1);
+  source.Add(2);
 
-  RepeatedType fields1(original);
-  ASSERT_EQ(2, fields1.size());
-  EXPECT_EQ(1, fields1.Get(0));
-  EXPECT_EQ(2, fields1.Get(1));
+  RepeatedField<int> destination(source);
 
-  RepeatedType fields2(token, nullptr, original);
-  ASSERT_EQ(2, fields1.size());
-  EXPECT_EQ(1, fields1.Get(0));
-  EXPECT_EQ(2, fields1.Get(1));
-}
-
-TEST(RepeatedField, CopyConstructCords) {
-  auto token = internal::InternalVisibilityForTesting{};
-  using RepeatedType = RepeatedField<absl::Cord>;
-  RepeatedType original;
-  original.Add(absl::Cord("hello"));
-  original.Add(absl::Cord("world and text to avoid SSO"));
-
-  RepeatedType fields1(original);
-  ASSERT_EQ(2, fields1.size());
-  EXPECT_EQ("hello", fields1.Get(0));
-  EXPECT_EQ("world and text to avoid SSO", fields1.Get(1));
-
-  RepeatedType fields2(token, nullptr, original);
-  ASSERT_EQ(2, fields1.size());
-  EXPECT_EQ("hello", fields1.Get(0));
-  EXPECT_EQ("world and text to avoid SSO", fields2.Get(1));
-}
-
-TEST(RepeatedField, CopyConstructIntegersWithArena) {
-  auto token = internal::InternalVisibilityForTesting{};
-  using RepeatedType = RepeatedField<int>;
-  RepeatedType original;
-  original.Add(1);
-  original.Add(2);
-
-  Arena arena;
-  alignas(RepeatedType) char mem[sizeof(RepeatedType)];
-  RepeatedType& fields1 = *new (mem) RepeatedType(token, &arena, original);
-  ASSERT_EQ(2, fields1.size());
-  EXPECT_EQ(1, fields1.Get(0));
-  EXPECT_EQ(2, fields1.Get(1));
-}
-
-TEST(RepeatedField, CopyConstructCordsWithArena) {
-  auto token = internal::InternalVisibilityForTesting{};
-  using RepeatedType = RepeatedField<absl::Cord>;
-  RepeatedType original;
-  original.Add(absl::Cord("hello"));
-  original.Add(absl::Cord("world and text to avoid SSO"));
-
-  Arena arena;
-  alignas(RepeatedType) char mem[sizeof(RepeatedType)];
-  RepeatedType& fields1 = *new (mem) RepeatedType(token, &arena, original);
-  ASSERT_EQ(2, fields1.size());
-  EXPECT_EQ("hello", fields1.Get(0));
-  EXPECT_EQ("world and text to avoid SSO", fields1.Get(1));
-
-  // Contract requires dtor to be invoked for absl::Cord
-  fields1.~RepeatedType();
+  ASSERT_EQ(2, destination.size());
+  EXPECT_EQ(1, destination.Get(0));
+  EXPECT_EQ(2, destination.Get(1));
 }
 
 TEST(RepeatedField, IteratorConstruct) {
@@ -863,9 +826,13 @@ TEST(RepeatedField, MoveAssign) {
         Arena::CreateMessage<RepeatedField<int>>(&arena);
     destination->Add(3);
     const int* source_data = source->data();
+    const int* destination_data = destination->data();
     *destination = std::move(*source);
     EXPECT_EQ(source_data, destination->data());
     EXPECT_THAT(*destination, ElementsAre(1, 2));
+    // This property isn't guaranteed but it's useful to have a test that would
+    // catch changes in this area.
+    EXPECT_EQ(destination_data, source->data());
     EXPECT_THAT(*source, ElementsAre(3));
   }
   {
@@ -1254,19 +1221,6 @@ TEST(RepeatedField, PoisonsMemoryOnAssign) {
 
 #endif
 
-TEST(RepeatedField, Cleanups) {
-  Arena arena;
-  auto growth = internal::CleanupGrowth(
-      arena, [&] { Arena::CreateMessage<RepeatedField<int>>(&arena); });
-  EXPECT_THAT(growth.cleanups, testing::IsEmpty());
-
-  void* ptr;
-  growth = internal::CleanupGrowth(arena, [&] {
-    ptr = Arena::CreateMessage<RepeatedField<absl::Cord>>(&arena);
-  });
-  EXPECT_THAT(growth.cleanups, testing::UnorderedElementsAre(ptr));
-}
-
 // ===================================================================
 // RepeatedPtrField tests.  These pretty much just mirror the RepeatedField
 // tests above.
@@ -1404,27 +1358,7 @@ TEST(RepeatedPtrField, ArenaAllocationSizesMatchExpectedValues) {
 }
 
 TEST(RepeatedPtrField, NaturalGrowthOnArenasReuseBlocks) {
-  using Rep = RepeatedPtrField<std::string>;
-  Arena arena;
-  std::vector<Rep*> values;
-
-  static constexpr int kNumFields = 100;
-  static constexpr int kNumElems = 1000;
-  for (int i = 0; i < kNumFields; ++i) {
-    values.push_back(Arena::CreateMessage<Rep>(&arena));
-    auto& field = *values.back();
-    for (int j = 0; j < kNumElems; ++j) {
-      field.Add("");
-    }
-  }
-
-  size_t expected =
-      values.size() * values[0]->Capacity() * sizeof(std::string*) +
-      sizeof(std::string) * kNumElems * kNumFields;
-  // Use a 2% slack for other overhead.
-  // If we were not reusing the blocks, the actual value would be ~2x the
-  // expected.
-  EXPECT_THAT(arena.SpaceUsed(), AllOf(Ge(expected), Le(1.02 * expected)));
+  CheckNaturalGrowthOnArenasReuseBlocks<RepeatedPtrField<std::string>>(true);
 }
 
 TEST(RepeatedPtrField, AddAndAssignRanges) {
@@ -1629,18 +1563,9 @@ TEST(RepeatedPtrField, AddAllocated) {
     field.Add()->assign("filler");
   }
 
-  const auto ensure_at_capacity = [&] {
-    while (field.size() < field.Capacity()) {
-      field.Add()->assign("filler");
-    }
-  };
-  const auto ensure_not_at_capacity = [&] { field.Reserve(field.size() + 1); };
-
-  ensure_at_capacity();
   int index = field.size();
 
   // First branch:  Field is at capacity with no cleared objects.
-  ASSERT_EQ(field.size(), field.Capacity());
   std::string* foo = new std::string("foo");
   field.AddAllocated(foo);
   EXPECT_EQ(index + 1, field.size());
@@ -1648,7 +1573,6 @@ TEST(RepeatedPtrField, AddAllocated) {
   EXPECT_EQ(foo, &field.Get(index));
 
   // Last branch:  Field is not at capacity and there are no cleared objects.
-  ensure_not_at_capacity();
   std::string* bar = new std::string("bar");
   field.AddAllocated(bar);
   ++index;
@@ -1657,7 +1581,6 @@ TEST(RepeatedPtrField, AddAllocated) {
   EXPECT_EQ(bar, &field.Get(index));
 
   // Third branch:  Field is not at capacity and there are no cleared objects.
-  ensure_not_at_capacity();
   field.RemoveLast();
   std::string* baz = new std::string("baz");
   field.AddAllocated(baz);
@@ -1666,7 +1589,9 @@ TEST(RepeatedPtrField, AddAllocated) {
   EXPECT_EQ(baz, &field.Get(index));
 
   // Second branch:  Field is at capacity but has some cleared objects.
-  ensure_at_capacity();
+  while (field.size() < field.Capacity()) {
+    field.Add()->assign("filler2");
+  }
   field.RemoveLast();
   index = field.size();
   std::string* moo = new std::string("moo");
@@ -1760,30 +1685,12 @@ TEST(RepeatedPtrField, Erase) {
 }
 
 TEST(RepeatedPtrField, CopyConstruct) {
-  auto token = internal::InternalVisibilityForTesting{};
   RepeatedPtrField<std::string> source;
   source.Add()->assign("1");
   source.Add()->assign("2");
 
-  RepeatedPtrField<std::string> destination1(source);
-  ASSERT_EQ(2, destination1.size());
-  EXPECT_EQ("1", destination1.Get(0));
-  EXPECT_EQ("2", destination1.Get(1));
+  RepeatedPtrField<std::string> destination(source);
 
-  RepeatedPtrField<std::string> destination2(token, nullptr, source);
-  ASSERT_EQ(2, destination2.size());
-  EXPECT_EQ("1", destination2.Get(0));
-  EXPECT_EQ("2", destination2.Get(1));
-}
-
-TEST(RepeatedPtrField, CopyConstructWithArena) {
-  auto token = internal::InternalVisibilityForTesting{};
-  RepeatedPtrField<std::string> source;
-  source.Add()->assign("1");
-  source.Add()->assign("2");
-
-  Arena arena;
-  RepeatedPtrField<std::string> destination(token, &arena, source);
   ASSERT_EQ(2, destination.size());
   EXPECT_EQ("1", destination.Get(0));
   EXPECT_EQ("2", destination.Get(1));
@@ -1824,47 +1731,6 @@ TEST(RepeatedPtrField, IteratorConstruct_Proto) {
   EXPECT_EQ(values[1].bb(), other.Get(1).bb());
 }
 
-TEST(RepeatedPtrField, SmallOptimization) {
-  // Properties checked here are not part of the contract of RepeatedPtrField,
-  // but we test them to verify that SSO is working as expected by the
-  // implementation.
-
-  // We use an arena to easily measure memory usage, but not needed.
-  Arena arena;
-  auto* array = Arena::CreateMessage<RepeatedPtrField<std::string>>(&arena);
-  EXPECT_EQ(array->Capacity(), 1);
-  EXPECT_EQ(array->SpaceUsedExcludingSelf(), 0);
-  std::string str;
-  auto usage_before = arena.SpaceUsed();
-  // We use UnsafeArenaAddAllocated just to grow the array without creating
-  // objects or causing extra cleanup costs in the arena to make the
-  // measurements simpler.
-  array->UnsafeArenaAddAllocated(&str);
-  // No backing array, just the string.
-  EXPECT_EQ(array->SpaceUsedExcludingSelf(), sizeof(str));
-  // We have not used any arena space.
-  EXPECT_EQ(usage_before, arena.SpaceUsed());
-  // Verify the string is where we think it is.
-  EXPECT_EQ(&*array->begin(), &str);
-  EXPECT_EQ(array->pointer_begin()[0], &str);
-  // The T** in pointer_begin points into the sso in the object.
-  EXPECT_TRUE(std::less_equal<void*>{}(array, &*array->pointer_begin()));
-  EXPECT_TRUE(std::less_equal<void*>{}(&*array->pointer_begin(), array + 1));
-
-  // Adding a second object stops sso.
-  std::string str2;
-  array->UnsafeArenaAddAllocated(&str2);
-  EXPECT_EQ(array->Capacity(), 3);
-  // Backing array and the strings.
-  EXPECT_EQ(array->SpaceUsedExcludingSelf(),
-            (1 + array->Capacity()) * sizeof(void*) + 2 * sizeof(str));
-  // We used some arena space now.
-  EXPECT_LT(usage_before, arena.SpaceUsed());
-  // And the pointer_begin is not in the sso anymore.
-  EXPECT_FALSE(std::less_equal<void*>{}(array, &*array->pointer_begin()) &&
-               std::less_equal<void*>{}(&*array->pointer_begin(), array + 1));
-}
-
 TEST(RepeatedPtrField, CopyAssign) {
   RepeatedPtrField<std::string> source, destination;
   source.Add()->assign("4");
@@ -1930,9 +1796,13 @@ TEST(RepeatedPtrField, MoveAssign) {
     RepeatedPtrField<std::string> destination;
     *destination.Add() = "3";
     const std::string* const* source_data = source.data();
+    const std::string* const* destination_data = destination.data();
     destination = std::move(source);
     EXPECT_EQ(source_data, destination.data());
     EXPECT_THAT(destination, ElementsAre("1", "2"));
+    // This property isn't guaranteed but it's useful to have a test that would
+    // catch changes in this area.
+    EXPECT_EQ(destination_data, source.data());
     EXPECT_THAT(source, ElementsAre("3"));
   }
   {
@@ -1945,9 +1815,13 @@ TEST(RepeatedPtrField, MoveAssign) {
         Arena::CreateMessage<RepeatedPtrField<std::string>>(&arena);
     *destination->Add() = "3";
     const std::string* const* source_data = source->data();
+    const std::string* const* destination_data = destination->data();
     *destination = std::move(*source);
     EXPECT_EQ(source_data, destination->data());
     EXPECT_THAT(*destination, ElementsAre("1", "2"));
+    // This property isn't guaranteed but it's useful to have a test that would
+    // catch changes in this area.
+    EXPECT_EQ(destination_data, source->data());
     EXPECT_THAT(*source, ElementsAre("3"));
   }
   {
@@ -2108,20 +1982,6 @@ TEST(RepeatedPtrField, DeleteSubrange) {
   // DeleteSubrange is a trivial extension of ExtendSubrange.
 }
 
-TEST(RepeatedPtrField, Cleanups) {
-  Arena arena;
-  auto growth = internal::CleanupGrowth(arena, [&] {
-    Arena::CreateMessage<RepeatedPtrField<std::string>>(&arena);
-  });
-  EXPECT_THAT(growth.cleanups, testing::IsEmpty());
-
-  growth = internal::CleanupGrowth(arena, [&] {
-    Arena::CreateMessage<RepeatedPtrField<TestAllTypes>>(&arena);
-  });
-  EXPECT_THAT(growth.cleanups, testing::IsEmpty());
-}
-
-
 // ===================================================================
 
 // Iterator tests stolen from net/proto/proto-array_unittest.
