@@ -1,9 +1,32 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
+// https://developers.google.com/protocol-buffers/
 //
-// Use of this source code is governed by a BSD-style
-// license that can be found in the LICENSE file or at
-// https://developers.google.com/open-source/licenses/bsd
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are
+// met:
+//
+//     * Redistributions of source code must retain the above copyright
+// notice, this list of conditions and the following disclaimer.
+//     * Redistributions in binary form must reproduce the above
+// copyright notice, this list of conditions and the following disclaimer
+// in the documentation and/or other materials provided with the
+// distribution.
+//     * Neither the name of Google Inc. nor the names of its
+// contributors may be used to endorse or promote products derived from
+// this software without specific prior written permission.
+//
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 // Author: jschorr@google.com (Joseph Schorr)
 //  Based on original Protocol Buffers design by
@@ -28,7 +51,6 @@
 #include "absl/log/absl_check.h"
 #include "absl/log/die_if_null.h"
 #include "absl/log/scoped_mock_log.h"
-#include "absl/strings/cord.h"
 #include "absl/strings/escaping.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
@@ -37,7 +59,6 @@
 #include "google/protobuf/io/tokenizer.h"
 #include "google/protobuf/io/zero_copy_stream_impl.h"
 #include "google/protobuf/map_unittest.pb.h"
-#include "google/protobuf/message.h"
 #include "google/protobuf/test_util.h"
 #include "google/protobuf/test_util2.h"
 #include "google/protobuf/unittest.pb.h"
@@ -74,7 +95,7 @@ class TextFormatTest : public testing::Test {
   static void SetUpTestSuite() {
     ABSL_CHECK_OK(File::GetContents(
         TestUtil::GetTestDataPath(
-            "google/protobuf/"
+            "third_party/protobuf/"
             "testdata/text_format_unittest_data_oneof_implemented.txt"),
         &static_proto_text_format_, true));
   }
@@ -95,7 +116,7 @@ class TextFormatExtensionsTest : public testing::Test {
  public:
   static void SetUpTestSuite() {
     ABSL_CHECK_OK(File::GetContents(
-        TestUtil::GetTestDataPath("google/protobuf/testdata/"
+        TestUtil::GetTestDataPath("third_party/protobuf/testdata/"
                                   "text_format_unittest_extensions_data.txt"),
         &static_proto_text_format_, true));
   }
@@ -141,134 +162,6 @@ TEST_F(TextFormatTest, ShortDebugString) {
 }
 
 
-TEST_F(TextFormatTest, ShortFormat) {
-  unittest::RedactedFields proto;
-  unittest::TestNestedMessageRedaction redacted_nested_proto;
-  unittest::TestNestedMessageRedaction unredacted_nested_proto;
-  redacted_nested_proto.set_optional_unredacted_nested_string("hello");
-  unredacted_nested_proto.set_optional_unredacted_nested_string("world");
-  proto.set_optional_redacted_string("foo");
-  proto.set_optional_unredacted_string("bar");
-  proto.add_repeated_redacted_string("1");
-  proto.add_repeated_redacted_string("2");
-  proto.add_repeated_unredacted_string("3");
-  proto.add_repeated_unredacted_string("4");
-  *proto.mutable_optional_redacted_message() = redacted_nested_proto;
-  *proto.mutable_optional_unredacted_message() = unredacted_nested_proto;
-  unittest::TestNestedMessageRedaction* redacted_message_1 =
-      proto.add_repeated_redacted_message();
-  unittest::TestNestedMessageRedaction* redacted_message_2 =
-      proto.add_repeated_redacted_message();
-  unittest::TestNestedMessageRedaction* unredacted_message_1 =
-      proto.add_repeated_unredacted_message();
-  unittest::TestNestedMessageRedaction* unredacted_message_2 =
-      proto.add_repeated_unredacted_message();
-  redacted_message_1->set_optional_unredacted_nested_string("5");
-  redacted_message_2->set_optional_unredacted_nested_string("6");
-  unredacted_message_1->set_optional_unredacted_nested_string("7");
-  unredacted_message_2->set_optional_unredacted_nested_string("8");
-  (*proto.mutable_map_redacted_string())["abc"] = "def";
-  (*proto.mutable_map_unredacted_string())["ghi"] = "jkl";
-
-  std::string value_replacement = "\\[REDACTED\\]";
-  EXPECT_THAT(google::protobuf::ShortFormat(proto),
-              testing::MatchesRegex(
-                  "optional_redacted_string: " +
-                  value_replacement +
-                  " "
-                  "optional_unredacted_string: \"bar\" "
-                  "repeated_redacted_string: " +
-                  value_replacement +
-                  " "
-                  "repeated_redacted_string: " +
-                  value_replacement +
-                  " "
-                  "repeated_unredacted_string: \"3\" "
-                  "repeated_unredacted_string: \"4\" "
-                  "optional_redacted_message: " +
-                  value_replacement +
-                  " "
-                  "optional_unredacted_message \\{ "
-                  "optional_unredacted_nested_string: \"world\" \\} "
-                  "repeated_redacted_message: " +
-                  value_replacement +
-                  " "
-                  "repeated_unredacted_message "
-                  "\\{ optional_unredacted_nested_string: \"7\" \\} "
-                  "repeated_unredacted_message "
-                  "\\{ optional_unredacted_nested_string: \"8\" \\} "
-                  "map_redacted_string: " +
-                  value_replacement +
-                  " "
-                  "map_unredacted_string \\{ key: \"ghi\" value: \"jkl\" \\}"));
-}
-
-TEST_F(TextFormatTest, Utf8Format) {
-  unittest::RedactedFields proto;
-  unittest::TestNestedMessageRedaction redacted_nested_proto;
-  unittest::TestNestedMessageRedaction unredacted_nested_proto;
-  redacted_nested_proto.set_optional_unredacted_nested_string(
-      "\350\260\267\346\255\214");
-  unredacted_nested_proto.set_optional_unredacted_nested_string(
-      "\350\260\267\346\255\214");
-  proto.set_optional_redacted_string("foo");
-  proto.set_optional_unredacted_string("bar");
-  proto.add_repeated_redacted_string("1");
-  proto.add_repeated_redacted_string("2");
-  proto.add_repeated_unredacted_string("3");
-  proto.add_repeated_unredacted_string("4");
-  *proto.mutable_optional_redacted_message() = redacted_nested_proto;
-  *proto.mutable_optional_unredacted_message() = unredacted_nested_proto;
-  unittest::TestNestedMessageRedaction* redacted_message_1 =
-      proto.add_repeated_redacted_message();
-  unittest::TestNestedMessageRedaction* redacted_message_2 =
-      proto.add_repeated_redacted_message();
-  unittest::TestNestedMessageRedaction* unredacted_message_1 =
-      proto.add_repeated_unredacted_message();
-  unittest::TestNestedMessageRedaction* unredacted_message_2 =
-      proto.add_repeated_unredacted_message();
-  redacted_message_1->set_optional_unredacted_nested_string("5");
-  redacted_message_2->set_optional_unredacted_nested_string("6");
-  unredacted_message_1->set_optional_unredacted_nested_string("7");
-  unredacted_message_2->set_optional_unredacted_nested_string("8");
-  (*proto.mutable_map_redacted_string())["abc"] = "def";
-  (*proto.mutable_map_unredacted_string())["ghi"] = "jkl";
-
-  std::string value_replacement = "\\[REDACTED\\]";
-  EXPECT_THAT(google::protobuf::Utf8Format(proto),
-              testing::MatchesRegex(
-                  "optional_redacted_string: " +
-                  value_replacement +
-                  "\n"
-                  "optional_unredacted_string: \"bar\"\n"
-                  "repeated_redacted_string: " +
-                  value_replacement +
-                  "\n"
-                  "repeated_redacted_string: " +
-                  value_replacement +
-                  "\n"
-                  "repeated_unredacted_string: \"3\"\n"
-                  "repeated_unredacted_string: \"4\"\n"
-                  "optional_redacted_message: " +
-                  value_replacement +
-                  "\n"
-                  "optional_unredacted_message \\{\n  "
-                  "optional_unredacted_nested_string: "
-                  "\"\xE8\xB0\xB7\xE6\xAD\x8C\"\n\\}\n"
-                  "repeated_redacted_message: " +
-                  value_replacement +
-                  "\n"
-                  "repeated_unredacted_message \\{\n  "
-                  "optional_unredacted_nested_string: \"7\"\n\\}\n"
-                  "repeated_unredacted_message \\{\n  "
-                  "optional_unredacted_nested_string: \"8\"\n\\}\n"
-                  "map_redacted_string: " +
-                  value_replacement +
-                  "\n"
-                  "map_unredacted_string \\{\n  "
-                  "key: \"ghi\"\n  value: \"jkl\"\n\\}\n"));
-}
-
 TEST_F(TextFormatTest, ShortPrimitiveRepeateds) {
   proto_.set_optional_int32(123);
   proto_.add_repeated_int32(456);
@@ -404,22 +297,6 @@ TEST_F(TextFormatTest, PrintUnknownFields) {
             message.DebugString());
 }
 
-TEST_F(TextFormatTest, PrintUnknownFieldsDeepestStackWorks) {
-  // Test printing of unknown fields in a message.
-
-  unittest::TestEmptyMessage message;
-  UnknownFieldSet* unknown_fields = message.mutable_unknown_fields();
-
-  for (int i = 0; i < 200; ++i) {
-    unknown_fields = unknown_fields->AddGroup(1);
-  }
-
-  unknown_fields->AddVarint(2, 100);
-
-  std::string str;
-  EXPECT_TRUE(TextFormat::PrintToString(message, &str));
-}
-
 TEST_F(TextFormatTest, PrintUnknownFieldsHidden) {
   // Test printing of unknown fields in a message when suppressed.
 
@@ -943,12 +820,6 @@ TEST_F(TextFormatTest, ParseBasic) {
   TestUtil::ExpectAllFieldsSet(proto_);
 }
 
-TEST_F(TextFormatTest, ParseCordBasic) {
-  absl::Cord cord(proto_text_format_);
-  TextFormat::ParseFromCord(cord, &proto_);
-  TestUtil::ExpectAllFieldsSet(proto_);
-}
-
 TEST_F(TextFormatExtensionsTest, ParseExtensions) {
   io::ArrayInputStream input_stream(proto_text_format_.data(),
                                     proto_text_format_.size());
@@ -1007,89 +878,6 @@ TEST_F(TextFormatTest, ParseUnknownEnumFieldProto3) {
   EXPECT_EQ(-2147483648, proto.repeated_nested_enum(3));
 }
 
-TEST_F(TextFormatTest, ErrorOnNoOpFieldsProto3) {
-  proto3_unittest::TestAllTypes proto;
-  TextFormat::Parser parser;
-  parser.ErrorOnNoOpFields(true);
-
-  {
-    const absl::string_view singular_int_parse_string = "optional_int32: 0";
-    EXPECT_TRUE(TextFormat::ParseFromString(singular_int_parse_string, &proto));
-    EXPECT_FALSE(parser.ParseFromString(singular_int_parse_string, &proto));
-  }
-  {
-    const absl::string_view singular_bool_parse_string = "optional_bool: false";
-    EXPECT_TRUE(
-        TextFormat::ParseFromString(singular_bool_parse_string, &proto));
-    EXPECT_FALSE(parser.ParseFromString(singular_bool_parse_string, &proto));
-  }
-  {
-    const absl::string_view singular_string_parse_string =
-        "optional_string: ''";
-    EXPECT_TRUE(
-        TextFormat::ParseFromString(singular_string_parse_string, &proto));
-    EXPECT_FALSE(parser.ParseFromString(singular_string_parse_string, &proto));
-  }
-  {
-    const absl::string_view nested_message_parse_string =
-        "optional_nested_message { bb: 0 } ";
-    EXPECT_TRUE(
-        TextFormat::ParseFromString(nested_message_parse_string, &proto));
-    EXPECT_FALSE(parser.ParseFromString(nested_message_parse_string, &proto));
-  }
-  {
-    const absl::string_view foreign_message_parse_string =
-        "optional_foreign_message { c: 0 } ";
-    EXPECT_TRUE(
-        TextFormat::ParseFromString(foreign_message_parse_string, &proto));
-    EXPECT_FALSE(parser.ParseFromString(foreign_message_parse_string, &proto));
-  }
-  {
-    const absl::string_view nested_enum_parse_string =
-        "optional_nested_enum: ZERO ";
-    EXPECT_TRUE(TextFormat::ParseFromString(nested_enum_parse_string, &proto));
-    EXPECT_FALSE(parser.ParseFromString(nested_enum_parse_string, &proto));
-  }
-  {
-    const absl::string_view foreign_enum_parse_string =
-        "optional_foreign_enum: FOREIGN_ZERO ";
-    EXPECT_TRUE(TextFormat::ParseFromString(foreign_enum_parse_string, &proto));
-    EXPECT_FALSE(parser.ParseFromString(foreign_enum_parse_string, &proto));
-  }
-  {
-    const absl::string_view string_piece_parse_string =
-        "optional_string_piece: '' ";
-    EXPECT_TRUE(TextFormat::ParseFromString(string_piece_parse_string, &proto));
-    EXPECT_FALSE(parser.ParseFromString(string_piece_parse_string, &proto));
-  }
-  {
-    const absl::string_view cord_parse_string = "optional_cord: '' ";
-    EXPECT_TRUE(TextFormat::ParseFromString(cord_parse_string, &proto));
-    EXPECT_FALSE(parser.ParseFromString(cord_parse_string, &proto));
-  }
-  {
-    // Sanity check that repeated fields work the same.
-    const absl::string_view repeated_int32_parse_string = "repeated_int32: 0 ";
-    EXPECT_TRUE(
-        TextFormat::ParseFromString(repeated_int32_parse_string, &proto));
-    EXPECT_TRUE(parser.ParseFromString(repeated_int32_parse_string, &proto));
-  }
-  {
-    const absl::string_view repeated_bool_parse_string =
-        "repeated_bool: false  ";
-    EXPECT_TRUE(
-        TextFormat::ParseFromString(repeated_bool_parse_string, &proto));
-    EXPECT_TRUE(parser.ParseFromString(repeated_bool_parse_string, &proto));
-  }
-  {
-    const absl::string_view repeated_string_parse_string =
-        "repeated_string: '' ";
-    EXPECT_TRUE(
-        TextFormat::ParseFromString(repeated_string_parse_string, &proto));
-    EXPECT_TRUE(parser.ParseFromString(repeated_string_parse_string, &proto));
-  }
-}
-
 TEST_F(TextFormatTest, ParseStringEscape) {
   // Create a parse string with escaped characters in it.
   std::string parse_string =
@@ -1189,7 +977,6 @@ TEST_F(TextFormatTest, ParseShortRepeatedWithTrailingComma) {
   parse_string = "repeated_nested_message: [ { bb: 1 }, ]";
   ASSERT_FALSE(TextFormat::ParseFromString(parse_string, &proto_));
   parse_string = "RepeatedGroup [{ a: 3 },]\n";
-  ASSERT_FALSE(TextFormat::ParseFromString(parse_string, &proto_));
 }
 
 TEST_F(TextFormatTest, ParseShortRepeatedEmpty) {
@@ -2492,6 +2279,58 @@ TEST(TextFormatUnknownFieldTest, TestUnknownExtension) {
   EXPECT_FALSE(parser.ParseFromString("unknown_field: 1", &proto));
 }
 
+template <bool silent, bool redact, bool randomize>
+class TextFormatBaseMarkerTest : public testing::Test {
+ public:
+  void SetUp() override {
+    saved_enable_debug_text_format_marker_ =
+        internal::enable_debug_text_format_marker;
+    saved_enable_debug_text_redaction_marker_ =
+        internal::enable_debug_text_redaction_marker;
+    saved_enable_debug_text_random_marker_ =
+        internal::enable_debug_text_random_marker;
+
+    internal::enable_debug_text_format_marker = silent;
+    internal::enable_debug_text_redaction_marker = redact;
+    internal::enable_debug_text_random_marker = randomize;
+  }
+
+  void TearDown() override {
+    internal::enable_debug_text_format_marker =
+        saved_enable_debug_text_format_marker_;
+    internal::enable_debug_text_redaction_marker =
+        saved_enable_debug_text_redaction_marker_;
+    internal::enable_debug_text_random_marker =
+        saved_enable_debug_text_random_marker_;
+  }
+
+ private:
+  bool saved_enable_debug_text_format_marker_;
+  bool saved_enable_debug_text_redaction_marker_;
+  bool saved_enable_debug_text_random_marker_;
+};
+
+using TextFormatRedactionTest = TextFormatBaseMarkerTest<false, true, false>;
+
+TEST_F(TextFormatRedactionTest, TestRedactedField) {
+  unittest::RedactedFields proto;
+  proto.set_optional_redacted_string("foo");
+  EXPECT_THAT(
+      proto.DebugString(),
+      testing::MatchesRegex("optional_redacted_string: \\[REDACTED\\]\n"));
+  EXPECT_THAT(
+      proto.ShortDebugString(),
+      testing::MatchesRegex("optional_redacted_string: \\[REDACTED\\]"));
+}
+
+TEST_F(TextFormatRedactionTest, TestTextFormatIsNotRedacted) {
+  unittest::RedactedFields proto;
+  proto.set_optional_redacted_string("foo");
+
+  std::string text;
+  ASSERT_TRUE(TextFormat::PrintToString(proto, &text));
+  EXPECT_EQ("optional_redacted_string: \"foo\"\n", text);
+}
 
 TEST(TextFormatFloatingPointTest, PreservesNegative0) {
   proto3_unittest::TestAllTypes in_message;
@@ -2511,7 +2350,6 @@ TEST(TextFormatFloatingPointTest, PreservesNegative0) {
             std::signbit(out_message.optional_double()));
 }
 
-
 }  // namespace text_format_unittest
 }  // namespace protobuf
 }  // namespace google
