@@ -28,23 +28,24 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#include <google/protobuf/util/type_resolver_util.h>
+#include "google/protobuf/util/type_resolver_util.h"
 
+#include <cstdint>
 #include <limits>
 #include <memory>
 #include <string>
 #include <vector>
 
-#include <google/protobuf/type.pb.h>
-#include <google/protobuf/wrappers.pb.h>
-#include <google/protobuf/map_unittest.pb.h>
-#include <google/protobuf/test_util.h>
-#include <google/protobuf/unittest.pb.h>
-#include <google/protobuf/unittest_custom_options.pb.h>
-#include <google/protobuf/util/json_format_proto3.pb.h>
-#include <google/protobuf/util/type_resolver.h>
-#include <google/protobuf/testing/googletest.h>
+#include "google/protobuf/type.pb.h"
+#include "google/protobuf/wrappers.pb.h"
+#include "google/protobuf/util/type_resolver.h"
+#include "google/protobuf/testing/googletest.h"
 #include <gtest/gtest.h>
+#include "google/protobuf/util/json_format_proto3.pb.h"
+#include "google/protobuf/map_unittest.pb.h"
+#include "google/protobuf/test_util.h"
+#include "google/protobuf/unittest.pb.h"
+#include "google/protobuf/unittest_custom_options.pb.h"
 
 namespace google {
 namespace protobuf {
@@ -75,13 +76,13 @@ class DescriptorPoolTypeResolverTest : public testing::Test {
         return &field;
       }
     }
-    return NULL;
+    return nullptr;
   }
 
   bool HasField(const Type& type, Field::Cardinality cardinality,
                 Field::Kind kind, const std::string& name, int number) {
     const Field* field = FindField(type, name);
-    if (field == NULL) {
+    if (field == nullptr) {
       return false;
     }
     return field->cardinality() == cardinality && field->kind() == kind &&
@@ -91,7 +92,7 @@ class DescriptorPoolTypeResolverTest : public testing::Test {
   bool CheckFieldTypeUrl(const Type& type, const std::string& name,
                          const std::string& type_url) {
     const Field* field = FindField(type, name);
-    if (field == NULL) {
+    if (field == nullptr) {
       return false;
     }
     return field->type_url() == type_url;
@@ -100,7 +101,7 @@ class DescriptorPoolTypeResolverTest : public testing::Test {
   bool FieldInOneof(const Type& type, const std::string& name,
                     const std::string& oneof_name) {
     const Field* field = FindField(type, name);
-    if (field == NULL || field->oneof_index() <= 0 ||
+    if (field == nullptr || field->oneof_index() <= 0 ||
         field->oneof_index() > type.oneofs_size()) {
       return false;
     }
@@ -109,7 +110,7 @@ class DescriptorPoolTypeResolverTest : public testing::Test {
 
   bool IsPacked(const Type& type, const std::string& name) {
     const Field* field = FindField(type, name);
-    if (field == NULL) {
+    if (field == nullptr) {
       return false;
     }
     return field->packed();
@@ -135,12 +136,12 @@ class DescriptorPoolTypeResolverTest : public testing::Test {
   }
 
   bool HasInt32Option(const RepeatedPtrField<Option>& options,
-                      const std::string& name, int32 value) {
+                      const std::string& name, int32_t value) {
     return HasOption<Int32Value>(options, name, value);
   }
 
   bool HasUInt64Option(const RepeatedPtrField<Option>& options,
-                       const std::string& name, uint64 value) {
+                       const std::string& name, uint64_t value) {
     return HasOption<UInt64Value>(options, name, value);
   }
 
@@ -158,7 +159,7 @@ class DescriptorPoolTypeResolverTest : public testing::Test {
     return false;
   }
 
-  string GetTypeUrl(std::string full_name) {
+  std::string GetTypeUrl(std::string full_name) {
     return kUrlPrefix + std::string("/") + full_name;
   }
 
